create table cal_L_Weekday(
  ID int not null,
  Token varchar(5) not null,
  Name varchar(35) not null,
  constraint calWeekday_PK primary key (ID)
);

create table cal_L_Month(
  ID int not null,
  Token varchar(5) not null,
  Name varchar(35) not null,
  constraint calMonth primary key (ID)
);

create table cal_S_Day(
  ID int not null,
  Date_ timestamp not null,
  WeekdayNo int not null,
  WeekYearNo int not null,
  WeekNo int not null,
  WeekToken varchar(10) not null,  -- z.B. '2013-08'
  DayInMonth int not null,
  MonthNo int not null,
  DayNo int not null,
  QuaterNo int not null,
  YearNo int not null,
  IsWorkDay smallint not null,
  IsWeekend smallint not null,
  IsHolyday smallint not null,
  IsBridgDay smallint not null,
  constraint calDay_PK primary key (ID),
  constraint calDay_FK_Weekday foreign key (WeekdayNo) references cal_L_Weekday(ID),
  constraint calDay_FK_Month foreign key (MonthNo) references cal_L_Month(ID)
);

create index calDay_SI_Weekday on cal_S_Day(WeekdayNo);
create unique index calDay_SI_Day on cal_S_Day(DayNo, YearNo);
create index calDay_SI_Week on cal_S_Day(WeekNo, YearNo);
create index calDay_SI_WeekYear on cal_S_Day(WeekNo, WeekYearNo);
create index calDay_SI_Month on cal_S_Day(MonthNo, YearNo);
create index calDay_SI_Year on cal_S_Day(YearNo);

create table cal_L_Time_Type(
  ID varchar(5) not null,
  Token varchar(5) not null,
  Name varchar(35) not null,
  Is_Typical_Worktime smallint default 0 not null,
  constraint calDatimeType_PK primary key (ID)
);

create table cal_L_Daytime(
  ID int not null,         -- Zeit als sprechende Zahl im Format 'hhmmss'
  T_HH timestamp not null,      -- Zeit im Format 'hh:00:00' (angefangene Stunde)
  T_HHMM timestamp not null,    -- Zeit im Format 'hh:mm:00' (angefangene Minute)
  T_HHMMSS timestamp not null,  -- Zeit im Format 'hh:mm:ss'
  DT timestamp not null,    -- Datum/Zeit ab Tag 0
  Hour_ int not null,       -- Stunde als Integer von T_HHMMSS
  Minute_ int not null,     -- Minute als Integer von T_HHMMSS
  Second_ int not null,     -- Sekunde als Integer von T_HHMMSS
  Abs_Hour int not null,   -- Anzahl Stunden seit ID = 0 (kann auch negativ werden)
  Abs_Minute int not null, -- Anzahl Minuten seit ID = 0 (kann auch negativ werden)
  Abs_Second int not null, -- Anzahl Sekunden seit ID = 0 (kann auch negativ werden)
  ID_Time_Type varchar(5),
  constraint calTime_PK primary key (ID),
  constraint calTime_FK_TimeType foreign key (ID_Time_Type) references cal_L_Time_Type(ID)
);

create index calDaytime_IX_TimeType on cal_L_Daytime (ID_Time_Type);
create unique index calDaytime_IX_AbsSecond on cal_L_Daytime (Abs_Second);

create table inv_K_VAT_Type (
  ID varchar(2) not null,
  Token varchar(30) not null,
  Description varchar(2048),
  constraint invVATType_PK primary key (ID)
);

create table inv_K_VAT (
  ID varchar(10) not null,
  ID_VATType varchar(2),
  Valid_Beg int not null,
  Valid_End int,
  Value_ decimal(18, 4) not null,
  constraint invVAT_PK primary key (ID),
  constraint invVAT_FK_invVATType foreign key (ID_VATType) references inv_K_VAT_Type (ID)
);

create index invVAT_SI_invVATType on inv_K_VAT (ID_VATType);

create table obj_L_Type(
  ID varchar(5) not null,
  Name varchar(50) not null,
  constraint objType_PK primary key (ID)
);

create table obj_S_Base (
  ID int not null,
  ID_Type varchar(5) not null,
  ID_VATType varchar(2),
  Number varchar(30) not null,
  Name varchar(50) not null,
  Description varchar(2048),
  constraint objBase_PK primary key (ID),
  constraint objBase_FK_Type foreign key (ID_Type) references obj_L_Type(ID),
  constraint objBase_FK_invVATType foreign key (ID_VATType) references inv_K_VAT_Type (ID)
);

create sequence objBase_PK_Sequence;

--$Include=Create ibeV2Dev Database DDL (Firebird) objBaseTriggerI.sql;

create index objBse_SI_Type on obj_S_Base(ID_Type);
create index objBase_SI_invVATType on obj_S_Base (ID_VATType);

create table obj_S_Inventory(
  ID int not null,
  Number_Serial varchar(30) not null,
  constraint objInventory_PK primary key (ID),
  constraint objInventory_FK_Base foreign key (ID) references obj_S_Base (ID)
);
