// ============================================================================
//
// Unit:    ibeDALConnectionADO
// Inhalt:  Formular
// Aufgabe: DAL Connection fr ADO Komponenten
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit ibeDALConnectionADO;

interface

uses
	SysUtils, Classes, DB, ADODB,
	ibeDALConnection, ibeDALQuery;

type

TibeDALConnectionADO = class(TibeDALConnection)
private
	Fq: TADOQuery;
	Function GetAdoDB: TADOConnection;
	Procedure SetAdoDB(aAdoDB: TADOConnection);
	Property AdoDB: TADOConnection Read GetAdoDB Write SetAdoDB;
protected
	property q: TADOQuery read Fq;
	function RetrieveConnectionString: string; virtual; abstract;
	procedure InternalSetup(); override;
  procedure InternalDAOQueryInit(a_daoQuery: TibeDALQuery); override;
  function GetDAOClass: TibeDALQueryClass; override;
  function GetActualKeepConnection(): Boolean; override;
  procedure SetActualKeepConnection(AKeep: Boolean); override;
  function GetActiveDatabase(): string; override;
  procedure SetActiveDatabase(const ADatabase: string); override;
public
	constructor Create(aOwner: TComponent); override;
	destructor Destroy; override;
  procedure Open; override;
  procedure Close; override;
  procedure DatasetInit(a_ds: TDataSet); override;
end;

implementation

uses
  ibeDALQueryADO;

constructor TibeDALConnectionADO.Create(aOwner: TComponent);
begin
	inherited Create(aOwner);

	Fq    := TADOQuery.Create(aOwner);
	AdoDB := TADOConnection.Create(aOwner);

  AdoDB.LoginPrompt := False;
end;

destructor TibeDALConnectionADO.Destroy;
begin
	inherited Destroy;

  AdoDB := nil;       // Connection in Fq durch nillen von AdoDB freigeben (siehe SetAdoDB())
	FreeAndNil(Fq);
end;

procedure TibeDALConnectionADO.InternalSetup();
begin
	AdoDB.ConnectionString := RetrieveConnectionString();
end;

procedure TibeDALConnectionADO.Open;
begin
  AdoDB.Connected := True;
end;

procedure TibeDALConnectionADO.Close;
begin
  AdoDB.Connected := False;
end;

procedure TibeDALConnectionADO.InternalDAOQueryInit(a_daoQuery: TibeDALQuery);
begin
  try
    DatasetInit(a_daoQuery.DataSet);
  except
    on E: Exception do begin
      raise EibeDALConnectionNotSupported.Create(Self, 'ADO DAOQueryInit: Ein DAO der Klasse ' + a_daoQuery.ClassName + ' kann nicht verarbeitet werden', E);
    end;
  end;
end;

function TibeDALConnectionADO.GetDAOClass: TibeDALQueryClass;
begin
  Result := TibeDALQueryADO;
end;

function TibeDALConnectionADO.GetActualKeepConnection(): Boolean; 
begin
  Result := q.Connection.KeepConnection;
end;

procedure TibeDALConnectionADO.SetActualKeepConnection(AKeep: Boolean); 
begin
  q.Connection.KeepConnection := AKeep;
end;

procedure TibeDALConnectionADO.DatasetInit(a_ds: TDataSet);
begin
  inherited DatasetInit(a_ds);

  if not (a_ds is TCustomADODataSet) then begin
    raise EibeDALConnectionNotSupported.Create(Self, 'ADO DatasetInit: die Methode darf nur fr Datasets der Klasse ' + TCustomADODataSet.ClassName + ' aufgerufen werden. bergeben wurde ein Dataset der Klasse ' + a_ds.ClassName + '.');
  end else begin
    TCustomADODataSet(a_ds).Connection := AdoDB;
  end;
end;

function TibeDALConnectionADO.GetActiveDatabase(): string;
begin
  Result := AdoDB.DefaultDatabase;
end;

procedure TibeDALConnectionADO.SetActiveDatabase(const ADatabase: string);
begin
  AdoDB.DefaultDatabase := ADatabase;
end;

Function TibeDALConnectionADO.GetAdoDB: TADOConnection;
begin
	Result := q.Connection;
end;

Procedure TibeDALConnectionADO.SetAdoDB(aAdoDB: TADOConnection);
begin
	if Assigned(q.Connection) then begin
		q.Connection.Free;
		q.Connection := nil;
	end;

	q.Connection := aAdoDB;
end;

end.

