// ============================================================================
//
// Unit:    ibeDALConnectionADO_MSSQL
// Inhalt:  Formular
// Aufgabe: DAL Connection fr MS SQL Server Datenbank mit ADO Komponenten
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit ibeDALConnectionADO_MSSQL;

interface

uses
	SysUtils, StrUtils, Classes,
	ibeTypes, ibeDALConnectionADO;

type

TibeDALConnectionADO_MSSQL = class(TibeDALConnectionADO)
private
protected
  function InternalDatabaseExists(const ADatabaseName: string): Boolean; override;
  procedure InternalDatabaseCreate(const ADatabaseName: string); override;
  procedure InternalDatabaseDrop(const ADatabaseName: string); override;
	procedure InternalTranStart(); override;
	procedure InternalTranCommit(); override;
	procedure InternalTranRollback(aOrigException: Exception = nil); override;
  function GetIsSupportedSequence(): Boolean; override;
  function GetIsSupportedIentityAfterInsert(): Boolean; override;
  function GetIsSupportedTempTable(): Boolean; override;
	function RetrieveConnectionString: string; override;
	function GetInitialDatabase(): string; override;
public
  function RetrieveIdentityValueAfterInsert(const AName: string = ''): Variant; override;
  function RetrieveSequenceValue(const AName: string = ''): Variant; override;
  function RetrieveTempTableExists(a_sTableName: WideString): Boolean; override;
  procedure RetrieveTableList(a_slTableNames: TStringList); override;
  function CVBoole(a_bValue: Boolean): string; override;
  function CVDate(a_dtDate: TDateTime): string; override;
  function CVDateTime(a_dtDate: TDateTime): string; override;
  function CVFloat(a_nValue: ibeFloat): string; override;
end;

implementation

uses
	ibeDALConnection;

function TibeDALConnectionADO_MSSQL.RetrieveConnectionString: string;
var sParams: string;
begin
  sParams := 'Provider=' + Parameters.Values['Provider'] + ';'
           + 'Integrated Security=' + Parameters.Values['Integrated Security'] + ';'
           + 'Persist Security Info=' + Parameters.Values['Persist Security Info'] + ';'
           + 'User ID=' + Parameters.Values['User ID'] + ';'
           + 'Initial Catalog=' + ActiveDatabase + ';'
           + 'Data Source=' + Parameters.Values['Data Source'] + ';'
           + 'Initial File Name=' + Parameters.Values['Initial File Name'] + ';'
           + 'Server SPN=' + Parameters.Values['Server SPN'];

  Result := sParams;
end;

function TibeDALConnectionADO_MSSQL.GetInitialDatabase(): string;
begin
  Result := Parameters.Values['Initial Catalog'];
end;

function TibeDALConnectionADO_MSSQL.InternalDatabaseExists(const ADatabaseName: string): Boolean;
begin
	q.Close;
	q.SQL.Text := 'select case when db_id(''' + ADatabaseName + ''') is not null then 1 else 0 end as DatabaseExists';
	q.Open();

  Result := q.FieldByName('DatabaseExists').AsInteger = 1;
  
	q.Close;
end;

procedure TibeDALConnectionADO_MSSQL.InternalDatabaseCreate(const ADatabaseName: string); 
begin
	q.Close;
	q.SQL.Text := 'create database ' + ADatabaseName;
	q.ExecSQL();
	q.Close;
end;

procedure TibeDALConnectionADO_MSSQL.InternalDatabaseDrop(const ADatabaseName: string); 
begin
  if DatabaseExists(ADatabaseName) then
  begin
    q.Close;
    Close();
    q.SQL.Text := 'drop database ' + ADatabaseName + #13#10;
    q.ExecSQL();
    q.Close;
  end;
end;

procedure TibeDALConnectionADO_MSSQL.InternalTranStart();
Begin
	q.Close;
	q.SQL.Text := 'begin tran';
	q.ExecSQL();
	q.Close;
end;

procedure TibeDALConnectionADO_MSSQL.InternalTranCommit();
begin
  q.Close;
  q.SQL.Text := 'commit tran';
  q.ExecSQL;
  q.Close;
end;

procedure TibeDALConnectionADO_MSSQL.InternalTranRollback(aOrigException: Exception = nil);
begin
  q.Close;
  Try
    q.SQL.Text := 'rollback tran';
    q.ExecSQL;
    q.Close;
  Except
    On E: Exception Do Begin
      If Assigned(aOrigException) Then Begin
        Raise ExceptClass(aOrigException.ClassType).Create(aOrigException.Message + #13#10
                                                         + 'Fehler in TranRollback(): ' + E.Message );
      End Else Begin
        Raise;
      End;
    End;
  End;
end;

function TibeDALConnectionADO_MSSQL.GetIsSupportedSequence(): Boolean; 
begin
  Result := True;
end;

function TibeDALConnectionADO_MSSQL.GetIsSupportedIentityAfterInsert(): Boolean; 
begin
  Result := True;
end;

function TibeDALConnectionADO_MSSQL.GetIsSupportedTempTable(): Boolean; 
begin
  Result := True;
end;

function TibeDALConnectionADO_MSSQL.RetrieveIdentityValueAfterInsert(const AName: string = ''): Variant;
begin
  q.Close;
	q.SQL.Text := 'select @@identity as IdentityValue';
	q.Open;
	Result := q.FieldByName('IdentityValue').Value;
	q.Close;
end;

function TibeDALConnectionADO_MSSQL.RetrieveSequenceValue(const AName: string = ''): Variant; 
begin
  q.Close;
	q.SQL.Text := 'select next value for ' + AName + ' as sequenceValue';
	q.Open;
	Result := q.FieldByName('sequenceValue').Value;
	q.Close;
end;

function TibeDALConnectionADO_MSSQL.RetrieveTempTableExists(a_sTableName: WideString): Boolean;
begin
	Result := a_sTableName <> '';

	if Result then begin
		q.Close;
		q.SQL.Text := 'select * from tempdb.sys.objects where object_id = object_id(N''tempdb..' + a_sTableName + ''') and type in (N''U'')';
		q.Open;

		Result := q.RecordCount > 0;
		q.Close;
  end;
end;

procedure TibeDALConnectionADO_MSSQL.RetrieveTableList(a_slTableNames: TStringList);
begin
  q.Close;
  q.SQL.Text := 'select name from sys.tables';
  q.Open;

  while not q.Eof do begin
    a_slTableNames.Add(q.FieldByName('Name').AsString);
    q.Next;
  end;
end;

function TibeDALConnectionADO_MSSQL.CVBoole(a_bValue: Boolean): string;
begin
  result := IfThen(a_bValue = True, '1', '0' );
end;

function TibeDALConnectionADO_MSSQL.CVDate(a_dtDate: TDateTime): string;
begin
	Result := FormatDateTime('yyyymmdd', a_dtDate);
end;

function TibeDALConnectionADO_MSSQL.CVDateTime(a_dtDate: TDateTime): string;
begin
	Result := FormatDateTime('yyyymmdd hh:nn:ss', a_dtDate);
end;

function TibeDALConnectionADO_MSSQL.CVFloat(a_nValue: ibeFloat): string;
var OrigDecimalSeperator, OrigThousandSeperator: Char;
begin
	OrigDecimalSeperator := DecimalSeparator;
	OrigThousandSeperator := ThousandSeparator;
	DecimalSeparator := '.';
	ThousandSeparator := ' ';
	Try
		Result := FloatToStr(a_nValue);
	Finally
		DecimalSeparator := OrigDecimalSeperator;
		ThousandSeparator := OrigThousandSeperator;
	End;
end;

initialization

TibeDALConnectionFactory.RegisterConnetion(gc_sConnectionType_ADO_MSSQL, TibeDALConnectionADO_MSSQL);

end.

