// ============================================================================
//
// Unit:    ibeDALQuery
// Inhalt:  Formular
// Aufgabe: DAL Query Basisklasse
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit ibeDALQuery;

interface

uses
  SysUtils, Classes, DB,
  ibeException;

type

TibeDALQuery = class
private
  FDataSet: TDataSet;
protected
  function GetRecordCount(): Integer; virtual; abstract;
  function GetActive: Boolean; virtual; abstract;
  procedure SetActive(a_bActive: Boolean); virtual; abstract;
  procedure InternalExecute; virtual; abstract;
  Function GetSQL: WideString; virtual; abstract;
  procedure SetSQL(const a_sSQL: WideString); virtual; abstract;
  function CreateDataset: TDataSet; virtual; abstract;
public
  constructor Create; virtual;
  destructor Destroy; override;
  procedure Open;
  procedure Close;
  procedure Execute;
  property DataSet: TDataSet read FDataSet;
  property RecordCount: Integer read GetRecordCount;
  property Active: Boolean read GetActive write SetActive;
  property SQL: WideString read GetSQL write SetSQL;
  procedure SQLLoadFromFile(const AFileName: TFileName);
end;

TibeDALQueryClass = class of TibeDALQuery;

EibeDALQuery = class(EibeException);

implementation

uses
  ibeFuncCommon;

constructor TibeDALQuery.Create;
begin
  inherited Create;

  FDataSet := CreateDataset;
end;

destructor TibeDALQuery.Destroy;
begin
  inherited Destroy;

  ibeFreeAndNil(FDataSet);
end;

procedure TibeDALQuery.Open;
begin
  Active := True;
end;

procedure TibeDALQuery.Close;
begin
  Active := False;
end;

procedure TibeDALQuery.Execute;
begin
  InternalExecute;
end;

procedure TibeDALQuery.SQLLoadFromFile(const AFileName: TFileName);
var
  LStatement: TStringList;
begin
  LStatement := TStringList.Create();
  try
    LStatement.LoadFromFile(AFileName);
    SQL := LStatement.Text;
  finally
    FreeAndNil(LStatement);
  end;
end;

end.
