// ============================================================================
//
// Unit:    ibeDALQueryADO
// Inhalt:  Formular
// Aufgabe: DAL Query basierend auf ADO Komponenten
//
//   (c) by Klaus Etscheid since 2016
// ============================================================================
unit ibeDALQueryADO;

interface

uses
  DB, ADODB,
  ibeDALQuery;

type

TibeDALQueryADO = class(TibeDALQuery)
private
  function GetQry: TADOQuery;
protected
  function GetRecordCount(): Integer; override;
  function GetActive: Boolean; override;
  procedure SetActive(a_bActive: Boolean); override;
  procedure InternalExecute; override;
  Function GetSQL: WideString; override;
  procedure SetSQL(const a_sSQL: WideString); override;
  function CreateDataset: TDataSet; override;
  property qry: TADOQuery read GetQry;
public
end;

implementation

function TibeDALQueryADO.GetRecordCount(): Integer; 
begin
  Result := qry.RecordCount;
end;

function TibeDALQueryADO.GetActive: Boolean;
begin
  Result := qry.Active;
end;

procedure TibeDALQueryADO.SetActive(a_bActive: Boolean);
begin
  qry.Active := a_bActive;
end;

procedure TibeDALQueryADO.InternalExecute;
begin
  qry.ExecSQL();
end;

Function TibeDALQueryADO.GetSQL: WideString;
begin
  Result := qry.SQL.Text;
end;

procedure TibeDALQueryADO.SetSQL(const a_sSQL: WideString);
begin
  qry.SQL.Text := a_sSQL;
end;

function TibeDALQueryADO.CreateDataset: TDataSet;
begin
  Result := TADOQuery.Create(nil);
end;

function TibeDALQueryADO.GetQry: TADOQuery;
begin
  if not (DataSet is TADOQuery) then begin
    raise EibeDALQuery.Create(Self, 'ADO Query Dataset: das Dataset ist von der Klasse ' + DataSet.ClassName + ' und kann nicht als TibeDALQuery angesprochen werden');
  end;

  Result := TADOQuery(DataSet);
end;

end.
