// ============================================================================
//
// Unit:    ibeDataObjectTypes
// Inhalt:  Formular
// Aufgabe: Typbibliothek fr DataObjects
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit ibeDataObjectTypes;

interface

uses
  SysUtils,
  ibeException;

type

TibeDataObjectState = (dosNone, dosBrowse, dosInsert, dosEdit);
TibeOrderByDirection = (obdAscending, obdDescending);

const DODefOrderByIDArray: array[TibeOrderByDirection] of string = ('ASC', 'DESC');

function DODefOrderByTypeFromString(const AOrderByID: string): TibeOrderByDirection;

type

TibeDODefDataType = (dodtString, dodtInteger, dodtIntDate, dodtDate, dodtTime, 
                     dodtDateTime, dodtBoolean, dodFloat, dodtMoney);

const DODefDataTypeIDArray: array[TibeDODefDataType] of string = ('String', 'Int', 'IntDate', 'Date', 'Time', 'DateTime', 'Boolean', 'Float', 'Money');

function DODefDataTypeFromString(const ADataType: string): TibeDODefDataType;

type

TibeDODefAggFunc = (doafNone, doafCount, doafSum, doafAvg, doafMin, doafMax);

TibeDOPKValueMode = (pkmCustom,     // die konkrete DO-Klasse generiert den PK
                     pkmUIDBin,     // 16 Byte Hex Zahl generiert vom DataObject
                     pkmUIDString,  // UID als 32 Zeichen Hex-String, generiert vom DataObject
                     pkmCounter,    // aus Countertabelle je TableName vom DataObject generiert
                     pkmSerial,     // vom DBMS gerneriert und vom DataObject ausgelesen
                     pkmNone);

const PKValueModeArray: array[TibeDOPKValueMode] of string = ('CUSTM', 'UIDBIN', 'UIDSTR', 'COUNTR', 'SERIL', '');

function PKValueModeTypeFromString(const APKValueModePKValueMode: string): TibeDOPKValueMode;

type

EibeDataObject = class(EibeException);
EibeDOFetchCountMismatch = class(EibeDataObject);

implementation

function DODefOrderByTypeFromString(const AOrderByID: string): TibeOrderByDirection;
begin
  if SameText(AOrderByID, DODefOrderByIDArray[obdAscending]) then
  begin
    Result := obdAscending;
  end else if SameText(AOrderByID, DODefOrderByIDArray[obdDescending]) then
  begin
    Result := obdDescending;
  end else
  begin
    raise EibeDataObject.Create('Nicht untersttze Sortierrichtung "' + AOrderByID + '"');
  end;
end;

function DODefDataTypeFromString(const ADataType: string): TibeDODefDataType;
var
  LDataType: TibeDODefDataType;
begin
  Result := dodtString;
  
  for LDataType := Low(TibeDODefDataType) to High(TibeDODefDataType) do
  begin
    if SameText(ADataType, DODefDataTypeIDArray[LDataType]) then
    begin
      Result := LDataType;
      Break;
    end;
  end;
end;                                                             

function PKValueModeTypeFromString(const APKValueModePKValueMode: string): TibeDOPKValueMode;
var
  LPKValueMode: TibeDOPKValueMode;
begin
  Result := pkmNone;
  
  for LPKValueMode := Low(TibeDOPKValueMode) to High(TibeDOPKValueMode) do
  begin
    if SameText(APKValueModePKValueMode, PKValueModeArray[LPKValueMode]) then
    begin
      Result := LPKValueMode;
      Break;
    end;
  end;
end;                                                             

end.
