//
// Unit:    ibeFCommonDOEdit
// Inhalt:  Formular
// Aufgabe: Bearbeitungsformular mit Datenzugriff aus DataObject
//
//   (c) by Klaus Etscheid since 2016
// ============================================================================
unit ibeFCommonDOEdit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, DB,
  Dialogs, ActnList, 
  ibeFCommonEdit, 
  ibeAppServiceProviderDB, ibeDALConnection, ibeDataObject, ibeMemTable;

type
  TibeFrmCommonDOEdit = class(TibeFrmCommonEdit)
    srcMain: TDataSource;
  private
    FDOMain: TibeDataObject;
    FDatMain: TibeMemTable;
		function GetASP: TibeAppServiceProviderDB;
    function GetConnection: TibeDALConnection;
    function GetDOMainClass: TibeDataObjectClass;
    procedure SetDOMainClass(ADOClass: TibeDataObjectClass);
  protected
    procedure DataLoad(); override;
    procedure DataDisplay(); override;
    procedure DataStore(); override;
    procedure DataSave(); override;
    procedure TransactionStart(); override;
    procedure TransactionCommit(); override;
    procedure TransactionRollback(); override;
    // Eigenschaften offenlegen
    property DatMain: TibeMemTable read FDatMain;
    property DOMain: TibeDataObject read FDOMain;
    property DOMainClass: TibeDataObjectClass read GetDOMainClass write SetDOMainClass;
  public
    constructor Create(AOwner: TComponent); override;
    property Connection: TibeDALConnection read GetConnection;
		property ASP: TibeAppServiceProviderDB read GetASP;
  end;

implementation

{$R *.dfm}

constructor TibeFrmCommonDOEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FDatMain := TibeMemTable.Create(Self);
  srcMain.DataSet := FDatMain;
end;

procedure TibeFrmCommonDOEdit.DataLoad();
begin
  if EditMode in [emBrowse, emEdit] then
  begin
    // den Datensatz laden
    DOMain.Fetch(PKValue);
    // das DataObject in Beatbeitungsmodus setzen
    DOMain.Edit();
  end else
  begin
    // das DO mit Insert ansteuern
    DOMain.Insert();
  end;

  // die Daten in die MemoryTable vom Bearbeitungsfenster kopieren
  DOMain.DataCopyToDataset(DatMain);
  // im Broese-Mode nur gucken, nicht anfassen
  DatMain.ReadOnly := EditMode = emBrowse;
end;

procedure TibeFrmCommonDOEdit.DataDisplay(); 
begin
end;

procedure TibeFrmCommonDOEdit.DataStore(); 
var
  LField: TField;
begin
  // auer dem PK-Feld alle Felder aus dem Dataset der Formulars in das 
  // DataObjekt kopieren
  for LField in DatMain.Fields do
  begin
    if not SameText(DOMain.PKName, LField.FieldName) then
    begin
      DOMain.Field[LField.FieldName].Value := LField.Value;
    end;
  end;
end;

procedure TibeFrmCommonDOEdit.DataSave();
begin
  DOMain.Post();
  PKValue := DOMain.PKValue;
end;

procedure TibeFrmCommonDOEdit.TransactionStart();
begin
  Connection.TranStart();
end;

procedure TibeFrmCommonDOEdit.TransactionCommit();
begin
  Connection.TranCommit();
end;

procedure TibeFrmCommonDOEdit.TransactionRollback();
begin
  Connection.TranRollback();
end;

function TibeFrmCommonDOEdit.GetDOMainClass: TibeDataObjectClass;
begin
  Result := TibeDataObjectClass(DOMain.ClassType);
end;

procedure TibeFrmCommonDOEdit.SetDOMainClass(ADOClass: TibeDataObjectClass);
begin
  if Assigned(DOMain) then
  begin
    FreeAndNil(FDOMain);
  end;

  FDOMain := ASP.DataObjectManager.CreateInstance(Connection, ADOClass);
end;

function TibeFrmCommonDOEdit.GetConnection: TibeDALConnection;
begin
  Result := ASP.Connection;
end;

function TibeFrmCommonDOEdit.GetASP: TibeAppServiceProviderDB;
begin
	Result := TibeAppServiceProviderDB(inherited ASP);
end;

end.

