// ============================================================================
//
// Unit:    ibeFCommonDOList
// Inhalt:  Formular
// Aufgabe: Basisformular vr Listen mit Datenherkunft aus einem DataObject
//
//   (c) by Klaus Etscheid since 2016
// ============================================================================
unit ibeFCommonDOList;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ActnList, StdCtrls, ExtCtrls, DB,
  ibeFCommonDataList, 
  ibeAppServiceProviderDB, ibeDALConnection, ibeDataObject;

type
  TibeFrmCommonDOList = class(TibeFrmCommonDataList)
  private
    FDOMain: TibeDataObject;
		function GetASP: TibeAppServiceProviderDB;
    function GetConnection: TibeDALConnection;
    function GetDOMainClass: TibeDataObjectClass;
    procedure SetDOMainClass(ADOClass: TibeDataObjectClass);
  protected
    procedure InternalActionRefresh(); override;
    procedure InternalActionDelete(); override;
    procedure DoBeforeRefresh(); override;
    property DOMain: TibeDataObject read FDOMain;
    property DOMainClass: TibeDataObjectClass read GetDOMainClass write SetDOMainClass;
  public
    destructor Destroy(); override;
    property Connection: TibeDALConnection read GetConnection;
		property ASP: TibeAppServiceProviderDB read GetASP;
  end;

implementation

{$R *.dfm}

destructor TibeFrmCommonDOList.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FDOMain);
end;

procedure TibeFrmCommonDOList.InternalActionRefresh();
begin
  inherited InternalActionRefresh();

  if Assigned(DOMain) then
  begin
    DOMain.Refresh();
    DOMain.DataCopyToDataset(DatMain);
  end;
end;

procedure TibeFrmCommonDOList.InternalActionDelete();
begin
  inherited InternalActionDelete();

  if Assigned(DOMain) then
  begin
    DOMain.Fetch(PKValue);
    DOMain.Delete();
  end;
end;

procedure TibeFrmCommonDOList.DoBeforeRefresh();
begin
  inherited DoBeforeRefresh();
  if Assigned(DOMain) then
  begin
    DOMain.FilterClear();
  end;
end;

function TibeFrmCommonDOList.GetDOMainClass: TibeDataObjectClass;
begin
  Result := TibeDataObjectClass(DOMain.ClassType);
end;

procedure TibeFrmCommonDOList.SetDOMainClass(ADOClass: TibeDataObjectClass);
begin
  if Assigned(DOMain) then
  begin
    FreeAndNil(FDOMain);
  end;

  FDOMain := ASP.DataObjectManager.CreateInstance(Connection, ADOClass);
  PKName := DOMain.PKName;
end;

function TibeFrmCommonDOList.GetConnection: TibeDALConnection;
begin
  Result := ASP.Connection;
end;

function TibeFrmCommonDOList.GetASP: TibeAppServiceProviderDB;
begin
	Result := TibeAppServiceProviderDB(inherited ASP);
end;

end.

