//
// Unit:    ibeFCommonEdit
// Inhalt:  Formular
// Aufgabe: Basisklasse fr Bearbeitungsformulare
//
//   (c) by Klaus Etscheid since 2016
// ============================================================================
unit ibeFCommonEdit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActnList, ExtCtrls, 
  ibeFCommon,
  ibeProtocol;

type

TibeEditMode = (emBrowse, emEdit, emNew);

  TibeFrmCommonEdit = class(TfrmCommon)
  private
    FPKValue: Variant;
    FEditMode: TibeEditMode;
  protected
    procedure InternalEdit();
    procedure DataLoad(); virtual; abstract;
    procedure DataDisplay(); virtual; abstract;
    procedure DataStore(); virtual; abstract;
    function DataValidate(var AMessage: string; AProtocol: TibeProtocolList): Boolean; virtual;
    procedure DataSave(); virtual; abstract;
    // Transaktionssteuerung, hier nur aufgerufen, muss von abgeleiteten Units
    // implementiert werden (sofern mglich!)
    procedure TransactionStart(); virtual;
    procedure TransactionCommit(); virtual;
    procedure TransactionRollback(); virtual;
    // Ereignisse
    procedure DoBeforeDataLoad(); virtual;
    procedure DoAfterDataLoad(); virtual;
    procedure DoBeforeDataDisplay(); virtual;
    procedure DoAfterDataDisplay(); virtual;
    procedure DoBeforeDataEdit(); virtual;
    procedure DoAfterDataEdit(); virtual;
    procedure DoBeforeDataStore(); virtual;
    procedure DoAfterDataStore(); virtual;
    procedure DoBeforeDataSave(); virtual;
    procedure DoAfterDataSave(); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    procedure ActionBrowse();
    procedure ActionEdit();
    procedure ActionNew();
    function ActionValidate(out AMessage: string; AProtocol: TibeProtocolList): Boolean;
    procedure ActionSave();
    procedure ActionCancel();
    property PKValue: Variant read FPKValue write FPKValue;
    property EditMode: TibeEditMode read FEditMode;
  end;

TibeFrmCommonEditClass = class of TibeFrmCommonEdit;
  
implementation

{$R *.dfm}

constructor TibeFrmCommonEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

procedure TibeFrmCommonEdit.ActionBrowse();
begin
  FEditMode := emBrowse;

  InternalEdit();
end;

procedure TibeFrmCommonEdit.ActionEdit();
begin
  FEditMode := emEdit;

  InternalEdit();
end;

procedure TibeFrmCommonEdit.ActionNew();
begin
  FEditMode := emNew;

  InternalEdit();
end;

procedure TibeFrmCommonEdit.InternalEdit();
var
  LOrigScreenCursor: TCursor;
begin
  LOrigScreenCursor := Screen.Cursor;
  try
    Screen.Cursor := crHourGlass;

    TransactionStart();
    try
      DoBeforeDataLoad();

      DataLoad();

      DoAfterDataLoad();

      DoBeforeDataDisplay();

      DataDisplay();

      DoAfterDataDisplay();

      DoBeforeDataEdit();

      TransactionCommit();
    except
      TransactionRollback();
      raise;
    end;
  finally
    Screen.Cursor := LOrigScreenCursor;
  end;
end;

function TibeFrmCommonEdit.ActionValidate(out AMessage: string; AProtocol: TibeProtocolList): Boolean;
var
  LOrigScreenCursor: TCursor;
begin
  LOrigScreenCursor := Screen.Cursor;
  try
    Screen.Cursor := crHourGlass;

    TransactionStart();
    try
      Result := DataValidate(AMessage, AProtocol);

      TransactionCommit();
    except
      TransactionRollback();
      raise;
    end;
  finally
    Screen.Cursor := LOrigScreenCursor;
  end;
end;

procedure TibeFrmCommonEdit.ActionSave();
var
  LMessage: string;
  LProtocol: TibeProtocolList;
  LOrigScreenCursor: TCursor;
begin
  LOrigScreenCursor := Screen.Cursor;
  try
    Screen.Cursor := crHourGlass;

    TransactionStart();
    try
      DoAfterDataEdit();

      DoBeforeDataStore();
  
      DataStore();

      DoAfterDataStore();
  
      LProtocol := TibeProtocolList.Create();
      try
        if not ActionValidate(LMessage, LProtocol) then
        begin
          raise EibeProtocolList.Create(Self, ClassName + '.Validate(): ' + LMessage, LProtocol);
        end;
      finally
        FreeAndNil(LProtocol);
      end;

      DoBeforeDataSave();

      DataSave();

      DoAfterDataSave();

      TransactionCommit();
    except
      TransactionRollback();
      raise;
    end;
  finally
    Screen.Cursor := LOrigScreenCursor;
  end;
end;

procedure TibeFrmCommonEdit.ActionCancel();
begin
end;

function TibeFrmCommonEdit.DataValidate(var AMessage: string; AProtocol: TibeProtocolList): Boolean;
begin
  // einen Standardtext fr die Fehlermeldung einsetzen
  AMessage := 'Es wurden Fehler festgestellt';
  // mit True als Ergebnis signalisieren, dass keine Fehler festgestellt wurden
  Result := True;
end;

procedure TibeFrmCommonEdit.TransactionStart();
begin
end;

procedure TibeFrmCommonEdit.TransactionCommit();
begin
end;

procedure TibeFrmCommonEdit.TransactionRollback();
begin
end;

procedure TibeFrmCommonEdit.DoBeforeDataLoad();
begin
end;

procedure TibeFrmCommonEdit.DoAfterDataLoad();
begin
end;

procedure TibeFrmCommonEdit.DoBeforeDataDisplay();
begin
end;

procedure TibeFrmCommonEdit.DoAfterDataDisplay();
begin
end;

procedure TibeFrmCommonEdit.DoBeforeDataEdit();
begin
end;

procedure TibeFrmCommonEdit.DoAfterDataEdit();
begin
end;

procedure TibeFrmCommonEdit.DoBeforeDataStore();
begin
end;

procedure TibeFrmCommonEdit.DoAfterDataStore();
begin
end;

procedure TibeFrmCommonEdit.DoBeforeDataSave();
begin
end;

procedure TibeFrmCommonEdit.DoAfterDataSave();
begin
end;

end.

