// ============================================================================
//
// Unit:    ibeFCommonList
// Inhalt:  Formular
// Aufgabe: Basisformular fr Listen
//
//   (c) by Klaus Etscheid since 2016
// ============================================================================
unit ibeFCommonList;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, VirtualTrees, ActnList, Menus,
  ibeVirtualTreeview,
  ibeFCommon, ibeFCommonEdit, ibeFCommonEditHost;

type
  TibeFrmCommonList = class(TfrmCommon)
    pnlMainTop: TPanel;
    pnlFilter: TPanel;
    pnlTitle: TPanel;
    pnlTitleBar: TPanel;
    lblRecordCountInfo: TLabel;
    lblTitel: TLabel;
    pnlMain: TPanel;
    pnlBottom: TPanel;
    mnuPopGridMain: TPopupMenu;
    actRefresh: TAction;
    Aktualisieren1: TMenuItem;
    actEdit: TAction;
    actNew: TAction;
    actDelete: TAction;
    procedure pnlFilterEnter(Sender: TObject);
    procedure pnlFilterExit(Sender: TObject);
    procedure actRefreshExecute(Sender: TObject);
    procedure actEditExecute(Sender: TObject);
    procedure actNewExecute(Sender: TObject);
    procedure actDeleteExecute(Sender: TObject);
  private
    FgrdMain: TibeVirtualTreeview;
    FFilterPanelFocused: Boolean;
    procedure MessageDialogKey(var Msg: TWMKey); message CM_DIALOGKEY;
  protected
    function GetPKValue(): Variant; virtual; abstract;
    procedure SetPKValue(APKValue: Variant); virtual; abstract;
    function GetGrdMainClass: TibeVirtualTreeviewClass; virtual;
    procedure FilterApply(); virtual;
    procedure ActionInitGridMain();
    procedure SelectNode(ANode: PVirtualNode);
    procedure InternalActionInitGridMain(); virtual;
    procedure InternalActionRefresh(); virtual;
    procedure InternalActionDisplay(); virtual;
    procedure InternalActionDelete(); virtual;
    procedure DoBeforeRefresh(); virtual;
    procedure DoAfterRefresh(); virtual;
    procedure DoBeforeEdit(AEditForm: TibeFrmCommonEdit); virtual;
    function AddGridMainColumn(const ACaption: string; AWidth: Integer): TVirtualTreeColumn; overload;
    function AddGridMainColumn(const ACaption: string; AWidth: Integer; AAlignment: TAlignment): TVirtualTreeColumn; overload;
    function RetrieveEditHostClass(): TibeFrmCommonEditHostClass; virtual; abstract;
    function RetrieveEditClass(): TibeFrmCommonEditClass; virtual; abstract;
    property grdMainClass: TibeVirtualTreeviewClass read GetGrdMainClass;
    property grdMain: TibeVirtualTreeview read FgrdMain;
    property FilterPanelFocused: Boolean read FFilterPanelFocused;
  public
    constructor Create(AOwner: TComponent); override;
    procedure ActionRefresh();
    procedure ActionDisplay();
    procedure ActionEdit(AEditMode: TibeEditMode);
    procedure ActionDelete();
    property PKValue: Variant read GetPKValue write SetPKValue;
  end;

implementation

{$R *.dfm}

constructor TibeFrmCommonList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFilterPanelFocused := False;

  FgrdMain := grdMainClass.Create(Self);
  grdMain.Parent := pnlMain;
  grdMain.Align := alClient;

  grdMain.PopupMenu := mnuPopGridMain;
  grdMain.HintMode := hmTooltip;
  grdMain.ShowHint := True;
  grdMain.EmptyListMessage := 'Keine Daten geladen';
  grdMain.LineStyle := lsSolid;

  grdMain.Colors.UnfocusedSelectionColor := clActiveCaption;
  grdMain.Colors.UnfocusedSelectionBorderColor := clActiveCaption;
  
  grdMain.Header.Options := grdMain.Header.Options + [hoVisible, hoDblClickResize];
  grdMain.TreeOptions.SelectionOptions := grdMain.TreeOptions.SelectionOptions + [toFullRowSelect];
  grdMain.TreeOptions.PaintOptions := grdMain.TreeOptions.PaintOptions 
                                    + [toShowHorzGridLines, toShowVertGridLines,
                                       toGhostedIfUnfocused]
                                    - [toShowTreeLines];

  ActionInitGridMain();
end;

procedure TibeFrmCommonList.ActionRefresh();
var
  LOrigScreenCursor: TCursor;
begin
  LOrigScreenCursor := Screen.Cursor;
  try
    Screen.Cursor := crHourGlass;
    
    DoBeforeRefresh();

    FilterApply();

    InternalActionRefresh();

    DoAfterRefresh();
    
    ActionDisplay();

    if not Assigned(grdMain.FocusedNode) then
    begin
      SelectNode(grdMain.GetFirst());
    end;
  finally
    Screen.Cursor := LOrigScreenCursor;
  end;
end;

procedure TibeFrmCommonList.ActionDisplay();
var
  LOrigScreenCursor: TCursor;
begin
  LOrigScreenCursor := Screen.Cursor;
  try
    Screen.Cursor := crHourGlass;

    grdMain.Clear();

    InternalActionDisplay();
  finally
    Screen.Cursor := LOrigScreenCursor;
  end;
end;

procedure TibeFrmCommonList.ActionEdit(AEditMode: TibeEditMode);
var
  LEditHostFormClass: TibeFrmCommonEditHostClass;
  LEditHostForm: TibeFrmCommonEditHost;
  LEditForm: TibeFrmCommonEdit;
begin
  LEditHostFormClass := RetrieveEditHostClass();

  LEditHostForm := LEditHostFormClass.Create(Self);
  try
    LEditHostForm.CreateEditForm(RetrieveEditClass(), LEditForm);

    LEditHostForm.PKValue := PKValue;

    DoBeforeEdit(LEditForm);

    case AEditMode of
      emBrowse: LEditHostForm.ActionBrowse();
      emEdit: LEditHostForm.ActionEdit();
      emNew: LEditHostForm.ActionNew();
    end;

    // Anzeige aktualisieren und auf den Datensatz aus dem Editformular 
    // positionieren
    ActionRefresh();
    PKValue := LEditHostForm.PKValue;
  finally
    FreeAndNil(LEditHostForm);
  end;
end;

procedure TibeFrmCommonList.DoBeforeEdit(AEditForm: TibeFrmCommonEdit);
begin
end;

procedure TibeFrmCommonList.ActionDelete();
begin
  InternalActionDelete();
  ActionRefresh();
end;

procedure TibeFrmCommonList.ActionInitGridMain();
begin
  grdMain.Clear();
  grdMain.Header.Columns.Clear();
  
  InternalActionInitGridMain();
end;

procedure TibeFrmCommonList.FilterApply();
begin
end;

procedure TibeFrmCommonList.SelectNode(ANode: PVirtualNode);
begin
  grdMain.FocusedNode := ANode;
  grdMain.Selected[ANode] := True;
  grdMain.Expanded[ANode] := True;
  grdMain.ScrollIntoView(ANode, True);
end;

procedure TibeFrmCommonList.DoBeforeRefresh();
begin
end;

procedure TibeFrmCommonList.DoAfterRefresh();
begin
end;

procedure TibeFrmCommonList.InternalActionInitGridMain();
begin
end;

procedure TibeFrmCommonList.InternalActionRefresh();
begin
end;

procedure TibeFrmCommonList.InternalActionDisplay();
begin
end;

procedure TibeFrmCommonList.InternalActionDelete();
begin
end;

function TibeFrmCommonList.AddGridMainColumn(const ACaption: string; AWidth: Integer): TVirtualTreeColumn;
begin
  Result := grdMain.Header.Columns.Add();

  Result.Text := ACaption;
  Result.Width := AWidth;
end;

function TibeFrmCommonList.AddGridMainColumn(const ACaption: string; AWidth: Integer; AAlignment: TAlignment): TVirtualTreeColumn;
begin
  Result := AddGridMainColumn(ACaption, AWidth);

  Result.Alignment := AAlignment;
end;

procedure TibeFrmCommonList.actDeleteExecute(Sender: TObject);
begin
  inherited;
  ActionDelete();
end;

procedure TibeFrmCommonList.actEditExecute(Sender: TObject);
begin
  inherited;
  ActionEdit(emEdit);
end;

procedure TibeFrmCommonList.actNewExecute(Sender: TObject);
begin
  inherited;
  ActionEdit(emNew);
end;

procedure TibeFrmCommonList.actRefreshExecute(Sender: TObject);
begin
  inherited;
  ActionRefresh();
end;

procedure TibeFrmCommonList.MessageDialogKey(var Msg: TWMKey); 
begin
  if (Msg.CharCode = VK_RETURN) and FilterPanelFocused then
  begin
    actRefresh.Execute();
    Msg.Result := 1;
  end else
  begin
    inherited;
  end;
end;

procedure TibeFrmCommonList.pnlFilterEnter(Sender: TObject);
begin
  inherited;
  FFilterPanelFocused := True;
end;

procedure TibeFrmCommonList.pnlFilterExit(Sender: TObject);
begin
  inherited;
  FFilterPanelFocused := False;
end;

function TibeFrmCommonList.GetGrdMainClass: TibeVirtualTreeviewClass; 
begin
  Result := TibeVirtualTreeview;
end;

end.

