// ============================================================================
//
// Unit:    ibeVirtualTreeview
// Inhalt:  Formular
// Aufgabe: VirtualStringTree mit Default-Funktionen
//
//   (c) by Klaus Etscheid since 2016
// ============================================================================
unit ibeVirtualTreeview;

interface

uses
  SysUtils, Classes, Graphics,
  VirtualTrees;

type

TibeVirtualTreeview = class(TVirtualStringTree)
private
  FDefaultTreeIndent: Integer;
  function GetDisplayAsTree(): Boolean;
  procedure SetDisplayAsTree(ADisplayAsTree: Boolean);
protected
  property DisplayAsTree: Boolean read GetDisplayAsTree write SetDisplayAsTree;
  property DefaultTreeIndent: Integer read FDefaultTreeIndent write FDefaultTreeIndent;
public
  constructor Create(AOwner: TComponent); override;
end;

TibeVirtualTreeviewClass = class of TibeVirtualTreeview;

implementation

constructor TibeVirtualTreeview.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  HintMode := hmTooltip;
  ShowHint := True;
  EmptyListMessage := 'Keine Daten geladen';
  LineStyle := lsSolid;

  TreeOptions.SelectionOptions := TreeOptions.SelectionOptions + [toFullRowSelect];

  Colors.UnfocusedSelectionColor := clActiveCaption;
  Colors.UnfocusedSelectionBorderColor := clActiveCaption;

  Header.Options := Header.Options + [hoVisible, hoDblClickResize];

  DefaultTreeIndent := Indent;
  DisplayAsTree := False;
end;

function TibeVirtualTreeview.GetDisplayAsTree(): Boolean;
begin
  Result := (toShowTreeLines in TreeOptions.PaintOptions) and (Indent <> 0);
end;

procedure TibeVirtualTreeview.SetDisplayAsTree(ADisplayAsTree: Boolean);
begin
  if ADisplayAsTree then
  begin
    Indent := DefaultTreeIndent;
    TreeOptions.PaintOptions := TreeOptions.PaintOptions + [toShowTreeLines];
  end else
  begin
    Indent := 0;
    TreeOptions.PaintOptions := TreeOptions.PaintOptions - [toShowTreeLines];
  end;
end;

end.

