// ============================================================================
//
// Unit:    ibeAppServiceProvider
// Inhalt:  Formular
// Aufgabe: Klassenbibliothek - Bereitstellen Anwendungsweit verfgbarer
//          Informationen und Services
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit ibeAppServiceProvider;

interface

uses
	SysUtils, IniFiles,
	ibeString, ibeFileUtil;

type

TibeAppServiceProvider = class
private
  FAppConfig: TIniFile;
  FAppExeName, FAppName: TFileName;
	FBinPath, FProgPath, FCfgPath, FRptPath, FDataPath, FAppConfigFileName: TFileName;
protected
  procedure SetAppExeName(const AAppExeName: TFileName); virtual;
	procedure SetBinPath(const path: TFileName); virtual;
  procedure SetProgPath(const path: TFileName); virtual;
	procedure SetCfgPath(const path: TFileName); virtual;
	procedure SetDataPath(const path: TFileName); virtual;
	procedure SetRptPath(const path: TFileName); virtual;
public
	constructor Create; virtual;
  destructor Destroy; override;
  property AppName: TFileName read FAppName;
  property AppExeName: TFileName read FAppExeName write SetAppExeName;
	property BinPath: TFileName read FBinPath write SetBinPath;
	property ProgPath: TFileName read FProgPath write SetProgPath;
	property CfgPath: TFileName read FCfgPath write SetCfgPath;
	property DataPath: TFileName read FDataPath write SetDataPath;
	property RptPath: TFileName read FRptPath write SetRptPath;
  property AppConfig: TIniFile read FAppConfig;
  property AppConfigFileName: TFileName read FAppConfigFileName write FAppConfigFileName;
end;

TibeAppServiceProviderClass = class of TibeAppServiceProvider;

function RetrieveASP: TibeAppServiceProvider;
procedure StoreASP(aASP: TibeAppServiceProvider);

implementation

uses
	ibeFuncCommon;

constructor TibeAppServiceProvider.Create;
begin
	inherited Create;

  AppConfigFileName := 'AppConfig.ini';
end;

destructor TibeAppServiceProvider.Destroy;
begin
  inherited Destroy;

  FreeAndNil(FAppConfig);
end;

procedure TibeAppServiceProvider.SetAppExeName(const AAppExeName: TFileName); 
begin
  FAppExeName := AAppExeName;
  FAppName := ExtractFileJustName(AAppExeName);
  BinPath := ExtractFilePath(AAppExeName);
end;

procedure TibeAppServiceProvider.SetBinPath(const path: TFileName);
begin
	FBinPath	:= path;
	ProgPath	:= RemoveLastWord(path, ['\']);
	DataPath	:= AddBackslash(ProgPath) + 'Data';
	CfgPath	  := AddBackslash(ProgPath) + 'Config';
	RptPath	  := AddBackslash(CfgPath)  + 'Report';
end;

procedure TibeAppServiceProvider.SetProgPath(const path: TFileName);
begin
  FProgPath := path;
end;

procedure TibeAppServiceProvider.SetCfgPath(const path: TFileName);
begin
  FCfgPath := path;

  if Assigned(FAppConfig) then begin
    ibeFreeAndNil(FAppConfig);
  end;
  FAppConfig := TIniFile.Create(AddBackslash(CfgPath) + AppConfigFileName);
end;

procedure TibeAppServiceProvider.SetDataPath(const path: TFileName);
begin
  FDataPath := path;
end;

procedure TibeAppServiceProvider.SetRptPath(const path: TFileName);
begin
  FRptPath := path;
end;

var
	m_ASPGlobal: TibeAppServiceProvider;

function RetrieveASP: TibeAppServiceProvider;
begin
	Result := m_ASPGlobal;
end;

procedure StoreASP(aASP: TibeAppServiceProvider);
begin
	if Assigned(m_ASPGlobal) then begin
		FreeAndNil(m_ASPGlobal);
	end;

	m_ASPGlobal := aASP;
end;

initialization

finalization

  FreeAndNil(m_ASPGlobal);

end.

