// ============================================================================
//
// Unit:    ibeCType
// Inhalt:  Formular
// Aufgabe: Funktionsbibliothek
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit ibeCType;

interface
(*
C Standard-Bibliothek
Tests fr Zeichenklassen: <ctype.h>

Die Definitionsdatei <ctype.h> vereinbart Funktionen zum Testen von Zeichen.
Jede Funktion hat ein int-Argument, dessen Wert entweder EOF ist oder als unsigned
char dargestellt werden kann, und der Resultatwert hat den Typ int. Die Funktionen
liefern einen von Null verschiedenen Wert (wahr), wenn das Argument c die beschriebene
Bedingung erfllt; andernfalls liefern sie Null.

isalnum(int c)	isalpha(c) oder isdigit(c) ist wahr
isalpha(int c)	isupper(c) oder islower(c) ist wahr
iscntrl(int c)	Steuerzeichen
isdigit(int c)	dezimale Ziffer
isgraph(int c)	sichtbares Zeichen, kein Leerzeichen
islower(int c)	Kleinbuchstabe (aber kein Umlaut oder )
isprint(int c)	sichtbares Zeichen, auch Leerzeichen
ispunct(int c)	sichtbares Zeichen, mit Ausnahme von Leerzeichen, Buchstabe oder Ziffer
isspace(int c)	Leerzeichen, Seitenvorschub (\f), Zeilentrenner (\n), Wagenrcklauf (\r), Tabulatorzeichen (\t), Vertikal-Tabulator (\v)
isupper(int c)	Grobuchstabe (aber kein Umlaut)
isxdigit(int c)	hexadezimale Ziffer

Im 7-Bit ASCII-Zeichensatz sind die sichtbaren Zeichen 0x20 (' ') bis 0x7E ('~');
die Steuerzeichen sind 0 (NUL) bis 0x1F (US) sowie 0x7F (DEL).
*)

uses
	SysUtils;

Const
	ibeCharsUpper				= ['A'..'Z'];
	ibeCharsLower				= ['a'..'z'];
	ibeCharsGerUpper		= ['', '', ''];
	ibeCharsGerLower		= ['', '', '', ''];
	ibeCharsDigit				= ['0'..'9'];
	ibeCharsXDigit			= ibeCharsDigit + ['x', 'a'..'f', 'X', 'A'..'F'];	// Hexadezimale Zahl
	ibeCharsWhiteSpace	= [#32, #10, #13, #12, #9, #11]; // Leerzeichen,
																											 // Seitenvorschub (\f),
																											 // Zeilentrenner (\n),
																											 // Wagenrcklauf (\r),
																											 // Tabulatorzeichen (\t),
																											 // Vertikal-Tabulator (\v)
	ibeCharsPunct				= ['.', ':', ',', ';', '!', '"', '', '$', '%', '&', '/', '(', ')',
												 '=', '?', '', '`', '^', '', '', '', '{', '}', '[', ']', '\',
												 '+', '*', '~', '''', '#', '-', '_', '<', '>'];

	ibeCharsGerUpperX		= ibeCharsUpper + ibeCharsGerUpper;
	ibeCharsGerLowerX		= ibeCharsLower + ibeCharsGerLower;
	ibeCharsGerUmlaut		= ibeCharsGerUpper + ibeCharsGerLower;

	ibeCharsAlpha				= ibeCharsGerUpperX + ibeCharsGerLowerX;
	ibeCharsAlNum				= ibeCharsAlpha + ibeCharsDigit;
	ibeCharsGraph				=	ibeCharsAlNum + ibeCharsPunct;
	ibeCharsPrint				= ibeCharsGraph + ibeCharsWhiteSpace;

Function IsAlNum( c: Char ): Boolean;       // isalpha(c) oder isdigit(c) ist wahr
Function IsAlpha( c: Char ): Boolean;       // isupper(c) oder islower(c) ist wahr
Function IsDigit( c: Char ): Boolean;       // dezimale Ziffer
Function IsGerLower( c: Char ): Boolean;    // Kleinbuchstabe (mit dt. Umlauten)
Function IsGerUmlaut( c: Char ): Boolean;   // Deutscher Umlaut
Function IsGerUpper( c: Char ): Boolean;    // Grobuchstabe (mit dt. Umlauten)
Function IsGraph( c: Char ): Boolean;       // sichtbares Zeichen, kein Leerzeichen
Function IsLower( c: Char ): Boolean;       // Kleinbuchstabe (aber kein Umlaut oder )
Function IsPrint( c: Char ): Boolean;       // sichtbares Zeichen, auch Leerzeichen
Function IsPunct( c: Char ): Boolean;				// sichtbares Zeichen, mit Ausnahme von Leerzeichen, Buchstabe oder Ziffer
Function IsSpace( c: Char ): Boolean;       // Leerzeichen, Seitenvorschub (\f), Zeilentrenner (\n), Wagenrcklauf (\r), Tabulatorzeichen (\t), Vertikal-Tabulator (\v)
Function IsUpper( c: Char ): Boolean;       // Grobuchstabe (aber kein Umlaut)
Function IsXDigit( c: Char ): Boolean;      // hexadezimale Ziffer

implementation

Function IsAlNum( c: Char ): Boolean;
Begin
// isalpha(c) oder isdigit(c) ist wahr
	Result := CharInSet(c, ibeCharsAlNum);
End;

Function IsAlpha( c: Char ): Boolean;
Begin
// isupper(c) oder islower(c) ist wahr
	Result := CharInSet(c, ibeCharsAlpha);
End;

Function IsDigit( c: Char ): Boolean;
Begin
// dezimale Ziffer
	Result := CharInSet(c, ibeCharsDigit);
End;

Function IsGerLower( c: Char ): Boolean;
Begin
// Kleinbuchstabe (mit dt. Umlauten)
	Result := CharInSet(c, ibeCharsGerLowerX);
End;

Function IsGerUmlaut( c: Char ): Boolean;
Begin
// Deutscher Umlaut
	Result := CharInSet(c, ibeCharsGerUmlaut);
End;

Function IsGerUpper( c: Char ): Boolean;
Begin
// Grobuchstabe (mit dt. Umlauten)
	Result := CharInSet(c, ibeCharsGerUpperX);
End;

Function IsGraph( c: Char ): Boolean;
Begin
// sichtbares Zeichen, kein Leerzeichen
	Result := CharInSet(c, ibeCharsGraph);
End;

Function IsLower( c: Char ): Boolean;
Begin
// Kleinbuchstabe (aber kein Umlaut oder )
	Result := CharInSet(c, ibeCharsLower);
End;

Function IsPrint( c: Char ): Boolean;
Begin
// sichtbares Zeichen, auch Leerzeichen
	Result := CharInSet(c, ibeCharsPrint);
End;

Function IsPunct( c: Char ): Boolean;
Begin
// sichtbares Zeichen, mit Ausnahme von Leerzeichen, Buchstabe oder Ziffer
	Result := CharInSet(c, ibeCharsPunct);
End;

Function IsSpace( c: Char ): Boolean;
Begin
// Leerzeichen, Seitenvorschub (\f), Zeilentrenner (\n), Wagenrcklauf (\r), Tabulatorzeichen (\t), Vertikal-Tabulator (\v)
	Result := CharInSet(c, ibeCharsWhiteSpace);
End;

Function IsUpper( c: Char ): Boolean;
Begin
// Grobuchstabe (aber kein Umlaut)
	Result := CharInSet(c, ibeCharsUpper);
End;

Function IsXDigit( c: Char ): Boolean;
Begin
// hexadezimale Ziffer
	Result := CharInSet(c, ibeCharsXDigit);
End;

end.
