// ============================================================================
//
// Unit:    ibeMemTable
// Inhalt:  Formular
// Aufgabe: Klassenbibliothek Memorytable als Ableitung von TClientDataset
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit ibeMemTable;

interface

uses
  DB, DBClient;

type

TibeMemTable = class(TClientDataSet)
private
protected
public
  destructor Destroy(); override;
  procedure Open();
  function LoadFromDataset(ASource: TDataSet; ACreateStructure: Boolean): Integer;
  procedure EmptyTable();
end;

implementation

destructor TibeMemTable.Destroy(); 
begin
  inherited Destroy();
end;

procedure TibeMemTable.Open();
begin
  if Active then
  begin
    Close();      
  end;

  CreateDataSet();
end;

function TibeMemTable.LoadFromDataset(ASource: TDataSet; ACreateStructure: Boolean): Integer;
var
  i: Integer;
  LSrcField, LDestField: TField;
begin
  if ACreateStructure then
  begin
    if Active then
    begin
      Close();
    end;
    Fields.Clear();
    FieldDefs.Clear();
    
    for LSrcField in ASource.Fields do
    begin
      LDestField := TFieldClass(LSrcField.ClassType).Create(nil);
      LDestField.FieldName := LSrcField.FieldName;
      LDestField.DisplayLabel := LSrcField.DisplayLabel;
      LDestField.Size := LSrcField.Size;
      LDestField.DataSet := Self;
      if LDestField is TNumericField then
      begin
        TNumericField(LDestField).DisplayFormat := TNumericField(LDestField).DisplayFormat;
      end else if LDestField is TBooleanField then
      begin
        TBooleanField(LDestField).DisplayValues := TBooleanField(LSrcField).DisplayValues;
      end;
    end;
  end;

  if not Active then
  begin
    Open();
  end;
  
  Result := 0;
  while not ASource.Eof do
  begin
    Append();
    for i := 0 to ASource.Fields.Count - 1 do
    begin
      Fields[i].Value := ASource.Fields[i].Value;
    end;
    Post();
    Inc(Result);
    ASource.Next();
  end;
end;

procedure TibeMemTable.EmptyTable;
begin
  if Active then begin
    First;

    while not Eof do begin
      Delete;
    end;
  end;
end;

end.
