// ============================================================================
//
// Unit:    ibeOSProcedures
// Inhalt:  Formular
// Aufgabe: Funktionsbibliothek zur Kapselung Betriebssystemabhngiger Methoden
//
//   (c) 2015 by Klaus Etscheid
// ============================================================================
unit ibeOSProcedures;

interface

Uses
{$IFDEF MSWINDOWS}
	Windows, ShellAPI,
{$ENDIF >>>>MSWINDOWS}
{$IFDEF LINUX}
{$ENDIF >>>>LINUX	}
	SysUtils, Math;

type

TExecuteCommandOption = (ecoHide, ecoShowNormal);

EEcecuteCommend = class(Exception);
  
function ExecuteAndWait(const aCommando: string): Integer;
procedure ExecuteCommand(const ACmd, AParams, APath: string; AOpt: TExecuteCommandOption);

Function GetTempPath: String;

function QueryElapsedSeconds(AStartSeconds: Extended = NaN): Extended;

implementation

{$IFDEF MSWINDOWS}

function ExecuteAndWait(const aCommando: string): Integer;
var
  tmpStartupInfo: TStartupInfo;
  tmpProcessInformation: TProcessInformation;
  tmpProgram: String;
  LWaitResult, dwExitCode: DWORD;
begin
  Result := 0;

  tmpProgram := trim(aCommando);
  FillChar(tmpStartupInfo, SizeOf(tmpStartupInfo), 0);
  with tmpStartupInfo do
  begin
    cb := SizeOf(TStartupInfo);
    wShowWindow := SW_HIDE;
  end;

  if CreateProcess(nil, pchar(tmpProgram), nil, nil, true, CREATE_NO_WINDOW,
    nil, nil, tmpStartupInfo, tmpProcessInformation) then
  begin
    // loop every 10 ms
    while WaitForSingleObject(tmpProcessInformation.hProcess, 10) > 0 do
    begin
      if GetExitCodeProcess(tmpProcessInformation.hProcess, dwExitCode) then
      begin
        Result := dwExitCode;
        
        if dwExitCode <> 0 then
        begin
          SetLastError(dwExitCode + $2000);
        end;
      end;
    end;

//    repeat
//      Inc(Result);
//      LWaitResult := WaitForSingleObject(tmpProcessInformation.hProcess, 10);
//    until LWaitResult > WAIT_OBJECT_0;
//
//    if GetExitCodeProcess(tmpProcessInformation.hProcess, dwExitCode) then
//    begin
//      if dwExitCode <> 0 then
//        SetLastError(dwExitCode + $2000);
//    end;

    CloseHandle(tmpProcessInformation.hProcess);
    CloseHandle(tmpProcessInformation.hThread);
  end else
  begin
    RaiseLastOSError;
  end;
end;

procedure ExecuteCommand(const ACmd, AParams, APath: string; AOpt: TExecuteCommandOption);
var
  LOpt: Integer;
  LPPath: Pointer;
  LResult: HINST;
begin
  case AOpt of
    ecoHide:        LOpt := SW_HIDE;
    ecoShowNormal:  LOpt := SW_SHOWNORMAL;
  else
    raise EEcecuteCommend.Create('ExecuteCommend("' + ACmd + '", ...) Nicht untersttzte Option #' + IntToStr(Ord(AOpt)));
  end;

  if APath <> '' then
  begin
    LPPath := @(APath[1]);
  end else
  begin
    LPPath := nil;
  end;
  
  LResult := ShellExecute(0, 'open', @(ACmd[1]), @(AParams[1]), LPPath, LOpt);

  if LResult <= 32 then
  begin
    raise EEcecuteCommend.Create('ExecuteCommend("' + ACmd + '", ...) wurde mit Fehlercode #' + IntToStr(LResult) + ' abgebrochen');
  end;
end;

Type

TWindowsPathFunction = function(nBufferLength: DWORD; lpBuffer: PChar): DWORD; stdcall;

Function GetWindowsPath(a_PathFunction: TWindowsPathFunction): string;
var pc: PChar;
	ps: DWord;
begin
	pc			:= nil;
	Result	:= '';
	try
		ps			:= MAX_PATH;
		pc			:= StrAlloc( ps );
		a_PathFunction( ps, pc );
		Result	:= StrPas( pc );
	finally
		if pc <> nil then StrDispose(pc);
	end;
End;

function GetTempPath: string;
begin
	Result := GetWindowsPath(Windows.GetTempPath);
end;

function ElapsedSecondsByPerformanceCounter(StartSeconds: Extended = NaN): Extended;
var Timer, Frequenzy: Int64;
begin
	QueryPerformanceFrequency(Frequenzy);
	QueryPerformanceCounter(Timer);

	Result := Timer / Frequenzy;

	if not IsNaN(StartSeconds) then begin
		Result := Result - StartSeconds;
	end;
end;

function ElapsedSecondsByTimerTics(StartSeconds: Extended = NaN): Extended;
begin
	Result := GetTickCount / 1000.0;

	if not IsNaN(StartSeconds) then begin
		Result := Result - StartSeconds;
	end;
end;

function QueryElapsedSeconds(AStartSeconds: Extended = NaN): Extended;
begin
  Result := ElapsedSecondsByPerformanceCounter(AStartSeconds);
end;

{$ENDIF >>>>MSWINDOWS}

end.
