// ============================================================================
//
// Unit:    ibeProtocol
// Inhalt:  Formular
// Aufgabe: Klassenbibliothek zum Erstellen eines Anwendungsprotokolls
//
//   (c) by Klaus Etscheid since 2016
// ============================================================================
unit ibeProtocol;

interface

uses
  SysUtils, Classes, Generics.Defaults, Generics.Collections,
  ibeException;

type

TibeProtocolEntryType = (petInformation, petWarning, petError);

TibeProtocolEntry = class(TObject)
private
  FEntryType: TibeProtocolEntryType;
  FEntryMessage: string;
protected
public
  property EntryType: TibeProtocolEntryType read FEntryType write FEntryType;
  property EntryMessage: string read FEntryMessage write FEntryMessage;
end;

TibeProtocolList = class(TObjectList<TibeProtocolEntry>)
private
protected
public
  procedure Assign(AProtocol: TibeProtocolList);
end;

EibeProtocolList = class(EibeException)
private
  FProtocol: TibeProtocolList;
protected
public
  constructor Create(ARaisingObject: TObject; const AMessage: string; AProtocol: TibeProtocolList); overload; virtual;
  destructor Destroy(); override;
  property Protocol: TibeProtocolList read FProtocol;
end;

implementation

procedure TibeProtocolList.Assign(AProtocol: TibeProtocolList);
var
  LEntry, LSourceEntry: TibeProtocolEntry;
begin
  for LSourceEntry in AProtocol do
  begin
    LEntry := TibeProtocolEntry.Create();
    LEntry.EntryType := LSourceEntry.EntryType;
    LEntry.EntryMessage := LSourceEntry.EntryMessage;

    Add(LEntry);
  end;
end;

constructor EibeProtocolList.Create(ARaisingObject: TObject; const AMessage: string; AProtocol: TibeProtocolList);
begin
  inherited Create(ARaisingObject, AMessage);

  FProtocol := TibeProtocolList.Create();
  FProtocol.Assign(AProtocol);
end;

destructor EibeProtocolList.Destroy();
begin
  inherited Destroy();

  FreeAndNil(FProtocol);
end;

end.

