// ============================================================================
//
// Unit:    ibeTypes
// Inhalt:  Formular
// Aufgabe: Typbibliothek
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit ibeTypes;

interface

Uses
	SysUtils, Math, DB, ADODB;

{ Datentypen }
Type

ibeInteger			= Integer;
ibeUnsignedInt	= Cardinal;
ibeFloat				= Extended;
ibeLongInteger	= Int64;
ibeString				= String;
ibeWideString		= WideString;
ibeCurrency			= Currency;

TibeSetOfChars	= TSysCharSet;
TibeSetOfBytes	= Set Of Byte;

Const

ibeMaxFloat				= MaxExtended;
ibeMinFloat				= MinExtended;
ibeFloatMaxExp		= +16384;	// grter Binaerexponent fr ibeFloat
ibeFloatMax10Exp	=  +4932;	// grter Exponent fr ibeFloat
ibeFloatMin10Exp	=  -4931;	// kleinster Exponent fr ibeFloat

Type

TibeDateTime	= TDateTime;

TibeDatatype	= (dtString, dtInteger, dtBoolean, dtFloat, dtDateTime, dtNone);
TibeSysCfgKey	= TibeDatatype;

Function DatatypeToName( dt: TibeDatatype ): String;

Type

TibeDateTimeUnit = (duSecond, duMinute, duQuaterHour, duHalfHour, duHour, duDay,
										duWeek, suMonth, duQuaterYear, duHalfYear, duYear, duNone);
TibeDateTimeUnits = Set Of TibeDateTimeUnit;

Const
	ibeAllTermDTUnits: TibeDateTimeUnits	= [duSecond, duMinute, duQuaterHour, duHalfHour,
																					 duHour, duDay, duWeek, suMonth, duQuaterYear,
																					 duHalfYear, duYear];
	ibeShortTermDTUnits: TibeDateTimeUnits	= [duSecond, duMinute, duQuaterHour, duHalfHour, duHour, duDay];
	ibeLongTermDTUnits: TibeDateTimeUnits	= [duDay, duWeek, suMonth, duQuaterYear, duHalfYear, duYear];

Function ibeConvertDTUnits( SrcDT: ibeInteger; SrcUnit, DestUnit: TibeDateTimeUnit ): ibeFloat;
Function ibeDTUnitToSecond( SrcDT: ibeFloat; SrcUnit: TibeDateTimeUnit ): ibeFloat;
Function ibeSecondToDTUnit( Second: ibeFloat; DestUnit: TibeDateTimeUnit ): ibeFloat;
Function ibeDTUnitToDateTime( SrcDT: ibeFloat; SrcUnit: TibeDateTimeUnit ): TibeDateTime;
Function ibeDateTimeToDTUnit( dt: TibeDateTime; DestUnit: TibeDateTimeUnit ): ibeFloat;

Function ibeDateTimeUnit2Name( dtu: TibeDateTimeUnit ): String;
Function ibeName2DateTimeUnit( Const Name: String ): TibeDateTimeUnit;
Function ibeDateTimeUnitNameList( Units: TibeDateTimeUnits ): String;

Const
	ibeSecondsInMinute			=  60.0;
	ibeSecondsInQuaterHour	=  15.0 * ibeSecondsInMinute;
	ibeSecondsInHalfHour		=  30.0 * ibeSecondsInMinute;
	ibeSecondsInHour				=  60.0 * ibeSecondsInMinute;
	ibeSecondsInDay					=  24.0 * ibeSecondsInHour;
	ibeSecondsInWeek				=   7.0 * ibeSecondsInDay;
	ibeSecondsInMonth				=  30.0 * ibeSecondsInDay;
	ibeSecondsInQuaterYear	=   3.0 * ibeSecondsInMonth;
	ibeSecondsInHalfYear		=   6.0 * ibeSecondsInMonth;
	ibeSecondsInYear				= 365.0 * ibeSecondsInDay;

	ibeMinutesInSecond			= 1.0 / ibeSecondsInMinute;
	ibeQuaterHoursInSecond	= 1.0 / ibeSecondsInQuaterHour;
	ibeHalfHoursInSecond		= 1.0 / ibeSecondsInHalfHour;
	ibeHoursInSecond				= 1.0 / ibeSecondsInHour;
	ibeDaysInSecond					= 1.0 / ibeSecondsInDay;
	ibeWeeksInSecond				= 1.0 / ibeSecondsInWeek;
	ibeMonthsInSecond				= 1.0 / ibeSecondsInMonth;
	ibeQuaterYearsInSecond	= 1.0 / ibeSecondsInQuaterYear;
	ibeHalfYearsInSecond   	= 1.0 / ibeSecondsInHalfYear;
	ibeYearsInSecond				= 1.0 / ibeSecondsInYear;

implementation

Function DatatypeToName( dt: TibeDatatype ): String;
Begin
	Case dt Of
		dtString:		Result := 'Text';
		dtInteger:	Result := 'Ganzzahl';
		dtBoolean:	Result := 'Wahr/Falsch';
		dtFloat:		Result := 'Reelle Zahl';
		dtDateTime:	Result := 'Datum/Zeit';
	Else					Result := 'Unbekannt';
	End;
End;

Function ibeConvertDTUnits( SrcDT: ibeInteger; SrcUnit, DestUnit: TibeDateTimeUnit ): ibeFloat;
Begin
	Result := ibeSecondToDTUnit( ibeDTUnitToSecond( SrcDT, SrcUnit ), DestUnit );
End;

Function ibeDTUnitToSecond( SrcDT: ibeFloat; SrcUnit: TibeDateTimeUnit ): ibeFloat;
Begin
	Case SrcUnit Of
		duSecond:			Result := SrcDT;
		duMinute:			Result := SrcDT * ibeSecondsInMinute;
		duQuaterHour:	Result := SrcDT * ibeSecondsInQuaterHour;
		duHalfHour:		Result := SrcDT * ibeSecondsInHalfHour;
		duHour:				Result := SrcDT * ibeSecondsInHour;
		duDay:				Result := SrcDT * ibeSecondsInDay;
		duWeek:				Result := SrcDT * ibeSecondsInWeek;
		suMonth:			Result := SrcDT * ibeSecondsInMonth;
		duQuaterYear:	Result := SrcDT * ibeSecondsInQuaterYear;
		duHalfYear:		Result := SrcDT * ibeSecondsInHalfYear;
		duYear:				Result := SrcDT * ibeSecondsInYear;
	Else						Result := 0.0;
	End;
End;

Function ibeSecondToDTUnit( Second: ibeFloat; DestUnit: TibeDateTimeUnit ): ibeFloat;
Begin
	Case DestUnit Of
		duSecond:			Result := Second;
		duMinute:			Result := Second * ibeMinutesInSecond;
		duQuaterHour:	Result := Second * ibeQuaterHoursInSecond;
		duHalfHour:		Result := Second * ibeHalfHoursInSecond;
		duHour:				Result := Second * ibeHoursInSecond;
		duDay:				Result := Second * ibeDaysInSecond;
		duWeek:				Result := Second * ibeWeeksInSecond;
		suMonth:			Result := Second * ibeMonthsInSecond;
		duQuaterYear:	Result := Second * ibeQuaterYearsInSecond;
		duHalfYear:		Result := Second * ibeHalfYearsInSecond;
		duYear:				Result := Second * ibeYearsInSecond;
	Else						Result := 0.0;
	End;
End;

Function ibeDTUnitToDateTime( SrcDT: ibeFloat; SrcUnit: TibeDateTimeUnit ): TibeDateTime;
Begin
	Result := ibeDTUnitToSecond( SrcDT, SrcUnit ) * ibeDaysInSecond;
End;

Function ibeDateTimeToDTUnit( dt: TibeDateTime; DestUnit: TibeDateTimeUnit ): ibeFloat;
Begin
	Result := ibeSecondToDTUnit( dt * ibeSecondsInDay, DestUnit );
End;

Function ibeDateTimeUnit2Name( dtu: TibeDateTimeUnit ): String;
Begin
	Case dtu Of
		duSecond:			Result := 'Sekunde(n)';
		duMinute:			Result := 'Minute(n)';
		duQuaterHour:	Result := '1/4 Stunde(n)';
		duHalfHour:		Result := '1/2 Stunde(n)';
		duHour:				Result := 'Stunde(n)';
		duDay:				Result := 'Tag(e)';
		duWeek:				Result := 'Woche(n)';
		suMonth:			Result := 'Monat(e)';
		duQuaterYear:	Result := '1/4 Jahr(e)';
		duHalfYear:		Result := '1/2 Jahr(e)';
		duYear:				Result := 'Jahr';
	Else						Result := 'Unbekannt';
	End;
End;

Function ibeName2DateTimeUnit( Const Name: String ): TibeDateTimeUnit;
Var u: TibeDateTimeUnit;
Begin
	Result := duNone;
	For u := duSecond To Pred(duNone) Do Begin
		If SameText( Name, ibeDateTimeUnit2Name( u ) ) Then Begin
			Result := u;
			Break;
		End;
	End;
End;

Function ibeDateTimeUnitNameList( Units: TibeDateTimeUnits ): String;
Var u: TibeDateTimeUnit;
Begin
	Result := '';
	For u := duSecond To Pred(duNone) Do Begin
		If u In Units Then Begin
			If Result <> '' Then Result := Result + #13#10;
			Result := Result + ibeDateTimeUnit2Name( u );
		End;
	End;
End;

end.

