// ============================================================================
//
// Unit:    objDOInventory
// Inhalt:  Formular
// Aufgabe: Datenobjekt fr Test-Inventar
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit objDOInventory;

interface

uses
  SysUtils,
  ibeDALConnection, ibeDataObject, ibeDataObjectTypes;

type

TobjDOInventory = class(TibeDataObject)
private
protected
  function DoBeforePost(): Boolean; override;
  procedure DoAfterDelete(APKValue: Variant); override;
  procedure InitNewRecord(); override;
public
end;

implementation

uses
  objDOBase;

function TobjDOInventory.DoBeforePost(): Boolean;
var
  LDOBase: TobjDOBase;
  LRecordPosted: Boolean;
begin
  LRecordPosted := inherited DoBeforePost();

  LDOBase := TobjDOBase.Create(Connection);
  try
    LDOBase.TraceEnabled := TraceEnabled;
    if State = dosInsert then
    begin
      LDOBase.Insert();
    end else if State = dosEdit then
    begin
      LDOBase.Fetch(Field[PKName].Value);
      LDOBase.Edit;
    end;

    LDOBase.Field['Obj_ID_Type'].Value := Field['Obj_ID_Type'].Value;
    LDOBase.Field['Obj_Number'].Value := Field['Obj_Number'].Value;
    LDOBase.Field['Obj_Name'].Value := Field['Obj_Name'].Value;
    LDOBase.Field['Obj_Description'].Value := Field['Obj_Description'].Value;
    LDOBase.Field['Obj_ID_VATType'].Value := Field['Obj_ID_VATType'].Value;

    LRecordPosted := LDOBase.Post() or LRecordPosted;

    if State = dosInsert then
    begin
      Field[PKName].Value := LDOBase.PKValue;
    end;
  finally
    FreeAndNil(LDOBase);
  end;

  Result := LRecordPosted;
end;

procedure TobjDOInventory.DoAfterDelete(APKValue: Variant); 
var
  LDOBase: TobjDOBase;
begin
  inherited DoAfterDelete(APKValue);

  LDOBase := TobjDOBase.Create(Connection);
  try
    LDOBase.TraceEnabled := TraceEnabled;
    LDOBase.Delete(APKValue);
  finally
    FreeAndNil(LDOBase);
  end;
end;

procedure TobjDOInventory.InitNewRecord();
begin
  inherited InitNewRecord();
  
  Field['Obj_ID_Type'].Value := 'INV'
end;

end.
