// ============================================================================
//
// Unit:    TestDAL_Firebird_IB
// Inhalt:  Formular
// Aufgabe: DUnit Test der DAL Klassen fr Firebird Datenbankserver mit 
//          InterBase Express Komponenten
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit TestDAL_Firebird_IB;

interface

uses
  SysUtils, Classes,
  ibeOSProcedures,
  TestFramework, 
  TestDataAccessObject,
  ibeDALConnection, ibeDALQuery;

type

TestTibeDAL_Firebird_IB = class(TestTibeDataAccessObject)
private
protected
  procedure DoBeforeInitScriptDDL(AConnection: TibeDALConnection); override;
public
  constructor Create(AMethodName: string); override;
  procedure SetUp(); override;
published
end;

implementation

constructor TestTibeDAL_Firebird_IB.Create(AMethodName: string); 
begin
  inherited Create(AMethodName);

  IBEConnectionName := 'TestDAL-Firebird-IB';
  DBName := 'Test_DAL';
  InitScriptDDLWithTransaction := True;

  InitScriptDDL := '..\Config\DBScript\Simple DDL Script (FB).sql';
  InitScriptDML := '..\Config\DBScript\Simple DML Script.sql';
end;

procedure TestTibeDAL_Firebird_IB.SetUp();
var
  LConnection: TibeDALConnection;
begin
  inherited SetUp();

  LConnection := ConnectionManager.RetriveConnection(nil, IBEConnectionName);
  try
    DBName := LConnection.ActiveDatabase;
  finally
    FreeAndNil(LConnection);
  end;
end;

// =================== [ BEG DoBeforeInitScriptDDL() ] =========================
// Variante von DoBeforeInitScriptDDL() bei der die leere Datenbank mit isql.exe
// und dynamisch erzeugtem SQL-Script mit create database Aufruf angelegt wird.
// Funktioniert nicht, weil isql.exe im Debug-Modus die Datenbank nur sporadisch
// und auerhalb vom Debugger gar nicht anlegt.
// Der Grund dafr ist nicht bekannt.
//
//procedure TestTibeDAL_Firebird_IB.DoBeforeInitScriptDDL(AConnection: TibeDALConnection); 
//var
//  i: Integer;
//  LCreateDBScript, LISQLExe: TFileName;
//  LCreateSQL: AnsiString;
//  LStream: TFileStream;
//begin
//  inherited DoBeforeInitScriptDDL(AConnection);
//
//  DBName := AConnection.Parameters.Values['Database'];
//
//  LCreateDBScript := '..\Data\DB\FB\TestCreateDB.sql';
//  LCreateDBScript := ExpandFileName(LCreateDBScript);
//
//  LCreateSQL := 'CREATE DATABASE "' + AnsiString(DBName) + '" user ''SYSDBA'' password ''masterkey'';';
//  
//  LStream := TFileStream.Create(LCreateDBScript, fmCreate);
//  try
//    LStream.Write(LCreateSQL[1], Length(LCreateSQL) * SizeOf(LCreateSQL[1]));
//  finally
//    FreeAndNil(LStream);
//  end;
//
//  if FileExists(DBName) then
//  begin
//    CheckTrue(DeleteFile(DBName), 'Datenbankdatei lschen');
//  end;
//  
//  //LISQLExe := 'C:\Program Files (x86)\Firebird\Firebird_2_5\bin\isql.exe';
//  LISQLExe := 'isql.exe';
//
//  ExecuteAndWait(LISQLExe + ' -i -b ' + LCreateDBScript);
//  //ExecuteCommand(LISQLExe, '-i ' + LCreateDBScript, '', ecoHide);
//
//  i := 0;
//  while not FileExists(DBName) and (i < 100) do
//  begin
//    Sleep(10);
//    Inc(i);
//  end;
//end;
// =================== [ END DoBeforeInitScriptDDL() ] =========================

// =================== [ BEG DoBeforeInitScriptDDL() ] =========================
// Variante von DoBeforeInitScriptDDL() bei der eine neue Datenbank mit kopieren
// einer im Config-Verzeichnis abgelegten leeren Datenbankdatei angelegt wird
//
//procedure TestTibeDAL_Firebird_IB.DoBeforeInitScriptDDL(AConnection: TibeDALConnection); 
//const
//  LEmptyDBName = '..\Config\Firebird\EmptyFBDatabase.fdb';
//var
//  LSrcStream, LDestStream: TFileStream;
//  LWriteSize: Integer;
//begin
//  inherited DoBeforeInitScriptDDL(AConnection);
//
//  if FileExists(DBName) then
//  begin
//    CheckTrue(DeleteFile(DBName), 'Datenbankdatei lschen');
//  end;
//
//  LSrcStream := TFileStream.Create(LEmptyDBName, fmOpenRead);
//  try
//    LDestStream := TFileStream.Create(DBName, fmCreate);
//    try
//      LWriteSize := LDestStream.CopyFrom(LSrcStream, LSrcStream.Size);
//
//      CheckEquals(LSrcStream.Size, LWriteSize, 'Fehler beim kopieren der leeren Datenbank: geschriebene Bytes stimmen nicht');
//    finally
//      FreeAndNil(LDestStream);
//    end;
//  finally
//    FreeAndNil(LSrcStream);
//  end;
//
//  CheckTrue(FileExists(DBName), 'Datenbankdatei "' + DBName + '" nicht gefunden.');
//end;
// =================== [ END DoBeforeInitScriptDDL() ] =========================

procedure TestTibeDAL_Firebird_IB.DoBeforeInitScriptDDL(AConnection: TibeDALConnection);
var
  LDBPath: TFileName;
begin
  inherited DoBeforeInitScriptDDL(AConnection);

  AConnection.DatabaseDrop(DBName);
  CheckFalse(FileExists(DBName), 'Datenbank "' + DBName + '" kann nicht gelscht werden.');

  LDBPath := ExtractFilePath(DBName);
  CheckTrue(ForceDirectories(LDBPath), 'Datenbankverzeichnis "' + LDBPath + '" kann nicht angelegt werden');
  AConnection.DatabaseCreate(DBName);
  CheckTrue(FileExists(DBName), 'Datenbankdatei "' + DBName + '" nicht gefunden.');
end;

initialization

  // Alle Testflle beim Test-Runner registrieren
  RegisterTest(TestTibeDAL_Firebird_IB.Suite);

end.
