// ============================================================================
//
// Unit:    TestDataObject
// Inhalt:  Formular
// Aufgabe: DUnit Test fr Basisklasse der Datenobjekte
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit TestDataObject;
{

  Delphi DUnit Testfall
  ----------------------
  Diese Unit enthlt ein Codegerst einer Testfallklasse, das vom Testfall-Experten
  erzeugt wurde. ndern Sie den erzeugten Code, damit die Methoden aus der 
  getesteten Unit korrekt eingerichtet und aufgerufen werden.

}

interface

uses
  TestFramework, SysUtils, Variants,
  ibeVariant,
  ibeDataObjectManager, ibeDataObject;

type
  // Testmethoden fr Klasse TibeDataObject
  
  TestTibeDataObject = class(TTestCase)
  strict private
    FibeDataObjectManager: TibeDataObjectManager;
  public
    procedure SetUp; override;
    procedure TearDown; override;
  published
    procedure TestRefreshObjBase;
    procedure TestDataObjectDeleteAll;
    procedure TestRefreshInvVAT;
    procedure TestTraverseDOObjBase;
    procedure TestFilterInvVAT;
    procedure TestAddViewInvVAT;
    procedure TestDataCopyToDataset;
    procedure TestDataObjectDayBasics;
  end;

implementation

uses
  ibeAppServiceProvider, ibeAppServiceProviderDB, ibeMemTable,
  objDOBase, invDOKVAT, calDOSDay;

procedure TestTibeDataObject.SetUp;
begin
  FibeDataObjectManager := TibeDataObjectManager.Create();
end;

procedure TestTibeDataObject.TearDown;
begin
  FibeDataObjectManager.Free;
  FibeDataObjectManager := nil;
end;

procedure TestTibeDataObject.TestRefreshObjBase;
var
  doObj: TobjDOBase;
begin
  doObj := TobjDOBase(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TobjDOBase));
  try
    CheckNotNull(doObj, 'Fehler beim Aufruf von CreateInstance fr DO objBase');

    doObj.Refresh();
    CheckNotEquals(0, doObj.RecordCount);
  finally
    FreeAndNil(doObj);
  end;
end;

procedure TestTibeDataObject.TestDataObjectDeleteAll;
var
  i, LCountToDelete, LDeletedCount: Integer;
  doObj: TobjDOBase;
begin
  doObj := TobjDOBase(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TobjDOBase));
  try
    CheckNotNull(doObj, 'Fehler beim Aufruf von CreateInstance fr DO objBase');

    for i := 0 to 4 do
    begin
      doObj.Insert();
      doObj.Field['Obj_ID_Type'].Value := 'ART';
      doObj.Field['Obj_Number'].Value := 'DeleteAll';
      doObj.Field['Obj_Name'].Value := 'DeleteAll ' + IntToStr(i);
      doObj.Field['Obj_Description'].Value := 'DeleteAll Target Number ' + IntToStr(i);
      doObj.Post();
    end;

    doObj.FilterClear();
    doObj.FilterAdd('Obj_Number', 'DeleteAll');
    doObj.Refresh();
    LCountToDelete := doObj.RecordCount;
    LDeletedCount := doObj.DeleteAll();
    CheckEquals(LCountToDelete, LDeletedCount, 'Anzahl zum Lschen selektierte Datenstze');
  finally
    FreeAndNil(doObj);
  end;
end;

procedure TestTibeDataObject.TestRefreshInvVAT;
var
  doVAT: TinvDOKVAT;
begin
  doVAT := TinvDOKVAT(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TinvDOKVAT));
  try
    CheckNotNull(doVAT, 'Fehler beim Aufruf von CreateInstance fr DO invKVAT');

    doVAT.Refresh();
    CheckNotEquals(0, doVAT.RecordCount);
    CheckEquals(EncodeDate(1900, 01, 01), doVAT.Field['Vat_Valid_Beg'].Value, 'Der erste Datensatz sollte Gltig Ab vom 01.01.1900 sein, geliefert wurde ' + VarAsString(doVAT.Field['Vat_Valid_Beg'].Value));
  finally
    FreeAndNil(doVAT);
  end;
end;

procedure TestTibeDataObject.TestFilterInvVAT;
var
  doVAT: TinvDOKVAT;
  LOK: Boolean;
begin
  doVAT := TinvDOKVAT(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TinvDOKVAT));
  try
    CheckNotNull(doVAT, 'Fehler beim Aufruf von CreateInstance fr DO invKVAT');
    doVAT.TraceEnabled := True;

    // Test der FilterAdd()
    //doVAT.Refresh();
    doVAT.FilterClear();
    doVAT.FilterAdd('Vat_ID', 'V0120v0');
    doVAT.Refresh();
    CheckEquals(1, doVAT.RecordCount);
    CheckEquals(EncodeDate(1978, 01, 01), doVAT.Field['Vat_Valid_Beg'].Value, 'FilterAdd(): der gefilterte Datensatz sollte Gltig Ab vom 01.01.1978 sein, geliefert wurde ' + VarAsString(doVAT.Field['Vat_Valid_Beg'].Value));

    // Filter Fetch()
    LOK := doVAT.FetchByDueDate('R1', EncodeDate(1979, 10, 20));
    CheckTrue(LOK, 'FetchByDueDate() sollte True liefern');
    CheckEquals(1, doVAT.RecordCount);
    CheckEquals(6.5, doVAT.Field['Vat_Value_'].Value, 'Der gefetchte reduzierte Steuersatz vom 20.10.1979');

    doVAT.FilterClear();
    doVAT.GroupByVatType();
    doVAT.Refresh();
    CheckEquals(3, doVAT.RecordCount, 'Gesamtanzahl Struerstze');
  finally
    FreeAndNil(doVAT);
  end;
end;

procedure TestTibeDataObject.TestAddViewInvVAT;
var
  doVAT: TinvDOKVAT;
begin
  doVAT := TinvDOKVAT(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TinvDOKVAT));
  try
    CheckNotNull(doVAT, 'Fehler beim Aufruf von CreateInstance fr DO invKVAT');
    doVAT.TraceEnabled := True;

    // ein GroupBy-Select dynamisch erstellen
    doVAT.AddViewGroupByVatType();
    doVAT.Refresh();
    CheckEquals(3, doVAT.RecordCount, 'Gesamtanzahl Steuerstze');

    // das dynamisch erstellte GroupBy-Select von oben weiter verwenden. Zustzlich
    // einen Filter auf alle vollen Steuerstze bis einschlielich dem 20.06.2015
    // einbauen. Geliefert werden sollte ein Datensatz, der neun Steuerstze beinhaltet
    // und der Maximalwert des Steuersatzes muss 19% sein
    doVAT.FilterClear();
    doVAT.FilterAdd(doVAT.FilterBuild('Vat_Valid_End', EncodeDate(2015, 06, 20), '<=') + ' or ' + doVAT.FilterBuild('Vat_Valid_End', Null));
    doVAT.FilterAddVatType('V0');
    doVAT.Refresh();
    CheckEquals(1, doVAT.RecordCount, 'Anzahl volle Steuerstze bis zum 20.06.2015');
    CheckEquals(9, doVAT.Field['VatType_Token_Count'].Value, 'Feldwert Count auslesen');
    CheckEquals(19, doVAT.Field['Vat_Value__Max'].Value, 'Feldwert Max auslesen');
  finally
    FreeAndNil(doVAT);
  end;
end;

procedure TestTibeDataObject.TestDataCopyToDataset;
var
  doVAT: TinvDOKVAT;
  datVAT: TibeMemTable;
begin
  doVAT := TinvDOKVAT(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TinvDOKVAT));
  try
    CheckNotNull(doVAT, 'Fehler beim Aufruf von CreateInstance fr DO invKVAT');

    // Steuerstze fr 2015 ermitteln und in ein Dataset kopieren. Im Dataset
    // mssen dann 3 Datenstze angekommen sein
    doVAT.FilterAddDueDate(EncodeDate(2015, 06, 20));
    doVAT.Refresh();

    datVAT := TibeMemTable.Create(nil);
    try
      doVAT.DataCopyToDataset(datVAT);
      CheckEquals(3, datVAT.RecordCount, 'Anzahl Datenstze in 2015');
      CheckTrue(datVAT.Locate('Vat_Value_', 0.0, []), 'Steuersatz 0% in 2015 nicht gefunden');
      CheckTrue(datVAT.Locate('Vat_Value_', 7.0, []), 'Steuersatz 7% in 2015 nicht gefunden');
      CheckTrue(datVAT.Locate('Vat_Value_', 19.0, []), 'Steuersatz 19% in 2015 nicht gefunden');
    finally
      FreeAndNil(datVAT);
    end;
  finally
    FreeAndNil(doVAT);
  end;
end;

procedure TestTibeDataObject.TestDataObjectDayBasics;
var
  doDay: TcalDOSDay;
begin
  doDay := TcalDOSDay(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TcalDOSDay));
  try
    CheckNotNull(doDay, 'Fehler beim Aufruf von CreateInstance fr DO calDOSDay');
    doDay.TraceEnabled := True;

    doDay.Refresh();
  finally
    FreeAndNil(doDay);
  end;
end;

procedure TestTibeDataObject.TestTraverseDOObjBase;
var
  i: Integer;
  doObj: TobjDOBase;
begin
  doObj := TobjDOBase(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TobjDOBase));
  try
    doObj.Refresh();

    i := 1;
    while doObj.Next() do
    begin
      Inc(i);
    end;
    CheckEquals(doObj.RecordCount, i, 'do.Next():');

    doObj.Last();
    i := 1;
    while doObj.Prior() do
    begin
      Inc(i);
    end;
    CheckEquals(doObj.RecordCount, i, 'do.Prior():');
  finally
    FreeAndNil(doObj);
  end;
end;

initialization

  // Alle Testflle beim Test-Runner registrieren
  RegisterTest(TestTibeDataObject.Suite);

end.
