// ============================================================================
//
// Unit:    TestDataObjectCascading
// Inhalt:  Formular
// Aufgabe: DUnit Test fr kaskadierende Operationen mit DataObjects
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit TestDataObjectCascading;
{

  Delphi DUnit Testfall
  ----------------------
  Diese Unit enthlt ein Codegerst einer Testfallklasse, das vom Testfall-Experten
  erzeugt wurde. ndern Sie den erzeugten Code, damit die Methoden aus der 
  getesteten Unit korrekt eingerichtet und aufgerufen werden.

}

interface

uses
  TestFramework, SysUtils, Variants,
  ibeVariant,
  ibeDataObjectManager, ibeDataObject;

type
  // Testmethoden fr Klasse TibeDataObject
  
  TestTibeDataObjectCascading = class(TTestCase)
  strict private
    FibeDataObjectManager: TibeDataObjectManager;
  public
    procedure SetUp; override;
    procedure TearDown; override;
  published
    procedure TestRefreshBasics();
    procedure TestAddInventory();
    procedure TestUpdateInventory();
  end;

implementation

uses
  ibeAppServiceProvider, ibeAppServiceProviderDB, ibeMemTable,
  objDOInventory;

procedure TestTibeDataObjectCascading.SetUp;
begin
  FibeDataObjectManager := TibeDataObjectManager.Create();
end;

procedure TestTibeDataObjectCascading.TearDown;
begin
  FibeDataObjectManager.Free;
  FibeDataObjectManager := nil;
end;

procedure TestTibeDataObjectCascading.TestRefreshBasics;
var
  doInv: TobjDOInventory;
begin
  doInv := TobjDOInventory(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TobjDOInventory));
  try
    CheckNotNull(doInv, 'Fehler beim Aufruf von CreateInstance fr DO objInventory');

    doInv.Refresh();
    CheckNotEquals(0, doInv.RecordCount);
  finally
    FreeAndNil(doInv);
  end;
end;

procedure TestTibeDataObjectCascading.TestAddInventory();
var
  doInv: TobjDOInventory;
begin
  doInv := TobjDOInventory(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TobjDOInventory));
  try
    CheckNotNull(doInv, 'Fehler beim Aufruf von CreateInstance fr DO objInventory');
    doInv.TraceEnabled := True;

    doInv.FilterAdd('Inv_Number_Serial', 'SNo.98765');
    doInv.Refresh();
    while doInv.RecordCount > 0 do
    begin
      doInv.Delete();
      doInv.FilterClear();
      doInv.FilterAdd('Inv_Number_Serial', 'SNo.98765');
      doInv.Refresh();
    end;
    doInv.FilterClear();
    
    doInv.Insert();
    doInv.Field['Inv_Number_Serial'].Value := 'SNo.98765';
    doInv.Field['Obj_Number'].Value := 12;
    doInv.Field['Obj_Name'].Value := 'Fahrrad';
    doInv.Field['Obj_Description'].Value := 'City-Bike frs Stdtchen';

    doInv.Post();
    doInv.Refresh();
    
    CheckNotEquals(0, doInv.RecordCount);
  finally
    FreeAndNil(doInv);
  end;
end;

procedure TestTibeDataObjectCascading.TestUpdateInventory();
var
  doInv: TobjDOInventory;
  LSerNo, LDatum: string;
  LRecordPosted: Boolean;
begin
  doInv := TobjDOInventory(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TobjDOInventory));
  try
    CheckNotNull(doInv, 'Fehler beim Aufruf von CreateInstance fr DO objInventory');
    doInv.TraceEnabled := True;

    LSerNo := 'SNo.12345';
    
    doInv.FilterAdd('Inv_Number_Serial', LSerNo);
    doInv.Refresh();
    while doInv.RecordCount > 0 do
    begin
      doInv.Delete();
      doInv.FilterClear();
      doInv.FilterAdd('Inv_Number_Serial', LSerNo);
      doInv.Refresh();
    end;

    doInv.FilterClear();
    
    doInv.Insert();
    doInv.Field['Inv_Number_Serial'].Value := LSerNo;
    doInv.Field['Obj_Number'].Value := -1;
    doInv.Field['Obj_Name'].Value := 'Hochdruckreiniger';
    doInv.Field['Obj_Description'].Value := 'Ein echter Krcher';
    //doInv.Field['Inv_Number_Serial'].Value := 'UPDATE-' + LDatum;
    doInv.Field['Inv_Number_Serial'].Value := LSerNo;
    LRecordPosted := doInv.Post();
    CheckTrue(LRecordPosted, 'Funktionsergebnis Datensatz einfgen');

    //doInv.FilterAdd('Inv_Number_Serial', 'UPDATE-' + LDatum);
    doInv.FilterAdd('Inv_Number_Serial', LSerNo);
    doInv.Refresh();
    CheckEquals(1, doInv.RecordCount, 'Inventat mit Seriennummer "' + LSerNo + '"');
    
    LDatum := '(' + DateToStr(Date()) + ')';
    doInv.Edit();
    doInv.Field['Obj_Number'].Value := DayOfWeek(Date());
    doInv.Field['Obj_Name'].Value := 'Hochdruckreiniger ' + LDatum;
    doInv.Field['Obj_Description'].Value := 'Ein echter Krcher ' + LDatum;
    //doInv.Field['Inv_Number_Serial'].Value := LSerNo;
    LRecordPosted := doInv.Post();
    CheckTrue(LRecordPosted, 'Funktionsergebnis Datensatz ndern');

    doInv.Refresh();
    CheckEquals(1, doInv.RecordCount, 'Inventat mit Seriennummer "' + LSerNo + '"');
    doInv.Edit();
    LRecordPosted := doInv.Post();
    CheckFalse(LRecordPosted, 'Funktionsergebnis von Post mit unvernderten Daten');
  finally
    FreeAndNil(doInv);
  end;
end;

initialization

  // Alle Testflle beim Test-Runner registrieren
  RegisterTest(TestTibeDataObjectCascading.Suite);

end.
