// ============================================================================
//
// Unit:    TestDataObjectInsertEdit
// Inhalt:  Formular
// Aufgabe: DUnit Test fr Insert-/Edit-Operationen mit dem DataObject
//
//   (c) by Klaus Etscheid since 2015
// ============================================================================
unit TestDataObjectInsertEdit;
{

  Delphi DUnit Testfall
  ----------------------
  Diese Unit enthlt ein Codegerst einer Testfallklasse, das vom Testfall-Experten
  erzeugt wurde. ndern Sie den erzeugten Code, damit die Methoden aus der 
  getesteten Unit korrekt eingerichtet und aufgerufen werden.

}

interface

uses
  TestFramework, SysUtils, Variants, DateUtils,
  ibeVariant, ibeFuncDateUtil,
  ibeDataObjectManager, ibeDataObject,
  calDOSDay;

type
  // Testmethoden fr Klasse TibeDataObject
  
  TestTibeDataObjectInsertEdit = class(TTestCase)
  strict private
    FibeDataObjectManager: TibeDataObjectManager;
    procedure AddDay(ADODay: TcalDOSDay; AIntDate: Integer); 
    procedure UpdateDay(ADODay: TcalDOSDay; AIntDate: Integer; ANewDate: TDateTime);
    procedure DeleteDay(ADODay: TcalDOSDay; AIntDate: Integer);
  public
    procedure SetUp; override;
    procedure TearDown; override;
  published
    procedure TestDataObjectDayBasics();
    procedure TestDataObjectDayInsert();
    procedure TestDataObjectDayUpdate();
    procedure TestDataObjectDayTransactions();
  end;

implementation

uses
  ibeAppServiceProvider, ibeAppServiceProviderDB, ibeMemTable;

procedure TestTibeDataObjectInsertEdit.SetUp;
begin
  FibeDataObjectManager := TibeDataObjectManager.Create();
end;

procedure TestTibeDataObjectInsertEdit.TearDown;
begin
  FibeDataObjectManager.Free;
  FibeDataObjectManager := nil;
end;

procedure TestTibeDataObjectInsertEdit.TestDataObjectDayBasics();
var
  doDay: TcalDOSDay;
begin
  doDay := TcalDOSDay(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TcalDOSDay));
  try
    CheckNotNull(doDay, 'Fehler beim Aufruf von CreateInstance fr DO calDOSDay');
    doDay.TraceEnabled := True;

    doDay.Refresh();
  finally
    FreeAndNil(doDay);
  end;
end;

procedure TestTibeDataObjectInsertEdit.TestDataObjectDayInsert();
var
  doDay: TcalDOSDay;
  LOrigKeepConnection: Boolean;
begin
  doDay := TcalDOSDay(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TcalDOSDay));
  try
    CheckNotNull(doDay, 'Fehler beim Aufruf von CreateInstance fr DO calDOSDay');

    LOrigKeepConnection := doDay.Connection.KeepConnection;
    CheckFalse(LOrigKeepConnection, 'Connection.KeepConnection');

    doDay.TraceEnabled := True;

    DeleteDay(doDay, 19630222);
    AddDay(doDay, 19630222);

    CheckEquals(LOrigKeepConnection, doDay.Connection.KeepConnection, 'Connection.Keepconnection nach DO.Post');
  finally
    FreeAndNil(doDay);
  end;
end;

procedure TestTibeDataObjectInsertEdit.TestDataObjectDayUpdate();
var
  doDay: TcalDOSDay;
begin
  doDay := TcalDOSDay(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TcalDOSDay));
  try
    CheckNotNull(doDay, 'Fehler beim Aufruf von CreateInstance fr DO calDOSDay');
    doDay.TraceEnabled := True;

    if not doDay.Fetch(19630222, True) then
    begin
      AddDay(doDay, 19630222);
    end;

    UpdateDay(doDay, 19630222, EncodeDate(1979, 06, 21));
  finally
    FreeAndNil(doDay);
  end;
end;

procedure TestTibeDataObjectInsertEdit.TestDataObjectDayTransactions();
var
  doDay: TcalDOSDay;
  LCtrlTran, LOrigKeepConnection: Boolean;
  LIntDate: Integer;
begin
 
  doDay := TcalDOSDay(FibeDataObjectManager.CreateInstance(RetrieveASP.Connection, TcalDOSDay));
  try
    CheckNotNull(doDay, 'Fehler beim Aufruf von CreateInstance fr DO calDOSDay');
    doDay.TraceEnabled := True;

    LOrigKeepConnection := RetrieveASP.Connection.ActualKeepConnection;
    CheckFalse(LOrigKeepConnection, 'ActualKeepConnection bei Testanfang');

    // -------------------------------------------------------------------------    
    // erster Durchlauf mit Rollback am Ende
    LIntDate := 20140713;  // WIR sind das 4. mal Fuball Weltmeister - Hurra! Hurra! Hurra!
    DeleteDay(doDay, LIntDate);

    LCtrlTran := doDay.Connection.TranStart();
    CheckTrue(LCtrlTran, 'Kontrolle ber Transaktion ist aktiv');
    CheckTrue(RetrieveASP.Connection.ActualKeepConnection, 'ActualKeepConnection bei nach TransaktionStart');

    doDay.Fetch(LIntDate, True);
      
    CheckEquals(doDay.RecordCount, 0, 'RecordCount vor AddDate()');
    AddDay(doDay, LIntDate);
    CheckTrue(RetrieveASP.Connection.ActualKeepConnection, 'ActualKeepConnection bei nach AddDay()');

    doDay.Fetch(LIntDate, True);
    CheckEquals(doDay.RecordCount, 1, 'RecordCount vor AddDate()');

    UpdateDay(doDay, LIntDate, IntDateToDate(19630222));
    CheckTrue(RetrieveASP.Connection.ActualKeepConnection, 'ActualKeepConnection bei nach UpdateDay()');

    doDay.Fetch(LIntDate, True);
    CheckEquals(doDay.RecordCount, 1, 'RecordCount nach UpdateDay()');
    
    RetrieveASP.Connection.TranRollback();
    CheckFalse(RetrieveASP.Connection.ActualKeepConnection, 'ActualKeepConnection nach TranCommit()');

    // Abschlieender Test fr Rollback: der Datensatz darf jetzt nicht mehr da sein
    doDay.Fetch(LIntDate, True);
    CheckEquals(doDay.RecordCount, 0, 'RecordCount nach TranRollback()');

    // -------------------------------------------------------------------------    
    // zweiter Durchlauf: die Transaktion wird Committed
    LIntDate := 19630222;
    LCtrlTran := doDay.Connection.TranStart();
    try
      CheckTrue(LCtrlTran, 'Kontrolle ber Transaktion ist aktiv');
      CheckTrue(RetrieveASP.Connection.ActualKeepConnection, 'ActualKeepConnection bei nach TransaktionStart');

      DeleteDay(doDay, LIntDate);

      doDay.Fetch(LIntDate, True);
      
      CheckEquals(doDay.RecordCount, 0, 'RecordCount vor AddDate()');
      AddDay(doDay, LIntDate);
      CheckTrue(RetrieveASP.Connection.ActualKeepConnection, 'ActualKeepConnection bei nach AddDay()');

      doDay.Fetch(LIntDate, True);
      CheckEquals(doDay.RecordCount, 1, 'RecordCount vor AddDate()');
 
      UpdateDay(doDay, LIntDate, Date());
      CheckTrue(RetrieveASP.Connection.ActualKeepConnection, 'ActualKeepConnection bei nach UpdateDay()');

      UpdateDay(doDay, LIntDate, IntDateToDate(LIntDate));
      CheckTrue(RetrieveASP.Connection.ActualKeepConnection, 'ActualKeepConnection bei nach UpdateDay()');

      doDay.Fetch(LIntDate, True);
      CheckEquals(doDay.RecordCount, 1, 'RecordCount nach UpdateDay()');
    
      RetrieveASP.Connection.TranCommit();
      CheckFalse(RetrieveASP.Connection.ActualKeepConnection, 'ActualKeepConnection nach TranCommit()');

      // Abschlieender Test fr Commit: jetzt muss der angelegte Tag noch vorhanden sein
      doDay.Fetch(LIntDate, True);
      CheckEquals(doDay.RecordCount, 1, 'RecordCount nach TranCommit()');
    except
      on E: Exception do
      begin
        RetrieveASP.Connection.TranRollback(E);
        CheckFalse(RetrieveASP.Connection.ActualKeepConnection, 'ActualKeepConnection nach TranRollback()');
        raise;
      end;
    end;
  finally
    FreeAndNil(doDay);
  end;
end;

procedure TestTibeDataObjectInsertEdit.AddDay(ADODay: TcalDOSDay; AIntDate: Integer);
var
  iYearOfWeek, iDayOfWeek, iMonth, iWeek: Word;
  LDate: TDateTime;
begin
  ADODay.FilterClear();
  ADODay.Insert();

  LDate := IntDateToDate(AIntDate);
  iDayOfWeek := DayOfTheWeek(LDate);
  iWeek := WeekOfTheYear(LDate, iYearOfWeek);
  iMonth := MonthOf(LDate);

  ADODay.Field['Day_ID'].Value := AIntDate;
  ADODay.Field['Day_Date_'].Value := LDate;
  ADODay.Field['Day_WeekdayNo'].Value := iDayOfWeek;
  ADODay.Field['Day_WeekNo'].Value := iWeek;
  ADODay.Field['Day_WeekYearNo'].Value := iYearOfWeek;
  ADODay.Field['Day_WeekToken'].Value := Format('%2.2d/%2.2d', [iWeek, iYearOfWeek Mod 100]);
  ADODay.Field['Day_DayNo'].Value := DayOfTheYear(LDate);
  ADODay.Field['Day_DayInMonth'].Value := DayOfTheMonth(LDate);
  ADODay.Field['Day_MonthNo'].Value := iMonth;
  ADODay.Field['Day_QuaterNo'].Value := (iMonth + 2) Div 3;
  ADODay.Field['Day_YearNo'].Value := YearOf(LDate);
  ADODay.Field['Day_IsWorkDay'].Value := iDayOfWeek in [1..5];
  ADODay.Field['Day_IsWeekend'].Value := iDayOfWeek in [6, 7];

  ADODay.Post();
end;

procedure TestTibeDataObjectInsertEdit.UpdateDay(ADODay: TcalDOSDay; AIntDate: Integer; ANewDate: TDateTime);
begin
  ADODay.Fetch(AIntDate);
  ADODay.Edit();

  ADODay.Field['Day_Date_'].Value := ANewDate;

  ADODay.Post();
end;

procedure TestTibeDataObjectInsertEdit.DeleteDay(ADODay: TcalDOSDay; AIntDate: Integer);
begin
  if ADODay.Fetch(AIntDate, True) then
  begin
    ADODay.Delete();
  end;
end;

initialization

  // Alle Testflle beim Test-Runner registrieren
  RegisterTest(TestTibeDataObjectInsertEdit.Suite);

end.
